#!/usr/bin/perl -w
# @(#)96------- 1.1- src/scripts/i386/ae/cfgeth.pl, vmi_cmd, isdvmi1_1 7/8/10 05:23:08 

use warnings;
use strict;
use Getopt::Long;

my %options = ();
my $IFCFGFILE = "/etc/sysconfig/network/ifcfg-";
my $ROUTEFILE = "/etc/sysconfig/network/routes";
my $HOSTNAMEFILE = "/etc/HOSTNAME";
my $HOSTFILE = "/etc/hosts";
my $DHCPCONFFILE = "/etc/dhcpd.conf";
my $DHCPDFILE = "/etc/sysconfig/dhcpd";
my $LEASEFILE = "/var/lib/dhcp/db/dhcpd.leases";

################################################################################
#  Get all possible parameters
################################################################################

if (! (GetOptions (\%options, 'enabled=s', 'interface=s','networktype=s','ipversion=s', 'iptype=s','ipv4address=s','netmask=s','ipv6address=s','startipaddr=s','endipaddr=s', 'subnet=s','router=s'))) {
    exit (1); 
}

################################################################################
#  Check parameters
################################################################################

sub checkOption
{
    my ($option, @validValue) = @_;
    if (!defined $options{$option}){
        print "AE: Property \"$option\" not specified. Ignore the setting.\n";
        exit 1;
    }
    my $attr = $options{$option};
    # if the option has no value list, return the value directly
    if (@validValue == 0) {
        return $attr;
    }
    for my $v (@validValue){
        if ($attr eq $v) {
            return $v;
        }
    }
    print "AE: Property \"$option\" value can only be @validValue.\n";
    exit 1;
}
################################################################################
# check if this interface is enabled, if "no", ignore the rest configuration
################################################################################

my $enabled=checkOption 'enabled', ('yes', 'no');
if ($enabled eq 'no'){
    print "AE: interface setting not enabled. Ignore the setting.\n";
    exit 0;
}

################################################################################
# get the network interface name
################################################################################

my $interface=checkOption 'interface', ('eth0', 'eth1', 'eth2', 'eth3');
$IFCFGFILE .= $interface;  # the ifcfg-eth* file


################################################################################
# The network can by confiugred as a standard network interface or a DHCP server
################################################################################

my $networktype=checkOption 'networktype', ('standard','dhcpserver');

# if the adapter is configured as a standard network interface
if ($networktype eq 'standard') {
# Configure one or both of IPv4 address and IPv6 addresses

     my $iptype=checkOption 'iptype', ('static','dynamic');
     if ($iptype eq 'dynamic') {
     # Configure the interface as a DHCP client
     my $ip_conf="BOOTPROTO='dhcp'\nIPADDR=\nMTU='1500'\nSTARTMODE='onboot'\nNETMASK=\n";
     open IFCFG, ">$IFCFGFILE" or die "AE: cannot open interface configuration file to write.\n";
     print IFCFG $ip_conf;
     close IFCFG;
     }else{
	   # Configure static Ip address

           my $ipv4_conf="";    
           my $ipv6_conf="";
           my $ipversion=checkOption 'ipversion', ('ipv4','ipv6','mix'); 
  	   if ($ipversion eq 'ipv4' || $ipversion eq 'mix') {
     	   # Configure static IPv4 address
                my $ipv4address=checkOption 'ipv4address';
                my $netmask=checkOption 'netmask';
                $ipv4_conf .= "BOOTPROTO='static'\nMTU='1500'\nSTARTMODE='onboot'\n"."IPADDR='$ipv4address'\nNETMASK='$netmask'\n";
                open IFCFG, ">$IFCFGFILE" or die "AE: cannot open interface configuration file to write.\n";
                print IFCFG $ipv4_conf;
                close IFCFG;
           }
           if ($ipversion eq 'ipv6' || $ipversion eq 'mix') {
           # Configure static IPv6 address
                my $ipv6address=checkOption 'ipv6address';
                if ($ipv4_conf eq ""){
		   # This line could be duplicate if static IPv4 address has been configured
                   $ipv6_conf.="BOOTPROTO='static'\nMTU='1500'\nSTARTMODE='onboot'\n";
                }else{
	           $ipv6_conf.=$ipv4_conf;
	        }
                my $i=0;
	        foreach my $addrv6 (split /,/,$ipv6address){
	           $i++;
		   $ipv6_conf.="IPADDR$i='$addrv6'\n";
	        }
                open IFCFG, ">$IFCFGFILE" or die "AE: cannot open interface configuration file to write.\n";
                print IFCFG $ipv6_conf;
                close IFCFG;
           }
     }
   
}
else{
     ############################
     #  configure DHCP server
     ###############################

     

     # Config /etc/dhcpd.conf file
     my $ipv4address=checkOption 'ipv4address';
     my $startipaddr=checkOption 'startipaddr';
     my $endipaddr=checkOption 'endipaddr';
     my $netmask=checkOption 'netmask';
     my $subnet=checkOption 'subnet';
     my $router=checkOption 'router';
     my $dhcp_server_conf="";
     $dhcp_server_conf.="# Licensed Internal Code - Property of IBM\n";
     $dhcp_server_conf.="#    A sample dhcpd.conf file\n";
     $dhcp_server_conf.="#  (Copy this to /etc/dhcpd.conf\n";
     $dhcp_server_conf.="#\n";
     $dhcp_server_conf.="# authoritative;\n";
     $dhcp_server_conf.="ddns-update-style none;\n";
     $dhcp_server_conf.="default-lease-time 600;\n";
     $dhcp_server_conf.="max-lease-time infinite;\n";
     $dhcp_server_conf.="subnet $subnet netmask $netmask {\n";
     $dhcp_server_conf.="option routers $router;\n";
     $dhcp_server_conf.="  range $startipaddr  $endipaddr;\n";
     $dhcp_server_conf.="}\n";
     open DHCPCFG, ">$DHCPCONFFILE" or die "AE: cannot open interface configuration file to write.\n";
     print DHCPCFG $dhcp_server_conf;
     close DHCPCFG;
     

     # Config /etc/syscofig/dhcpd
     `cat /etc/sysconfig/dhcpd | sed "s/^DHCPD_INTERFACE=\".*\"/DHCPD_INTERFACE=\"${interface}\"/g"  > /etc/sysconfig/dhcpd.tmp`;
     `cp /etc/sysconfig/dhcpd.tmp /etc/sysconfig/dhcpd`;
     `rm /etc/sysconfig/dhcpd.tmp`;
     # Make sure the leases file is there before we try to start
     unless (-e $LEASEFILE){
	     `touch ${LEASEFILE}`;
     }
     my $ipv4_conf = "BOOTPROTO='static'\nMTU='1500'\nSTARTMODE='onboot'\n"."IPADDR='$ipv4address'\nNETMASK='$netmask'\n";
     open IFCFG, ">$IFCFGFILE" or die "AE: cannot open interface configuration file to write.\n";
     print IFCFG $ipv4_conf;
     close IFCFG;
}

exit 0;

