#!/usr/bin/perl -w
# %Z%%M%------- %I%- %W% %G% %U% 
# this script should dependent on the isdvmi-mgmt rpm
use warnings;
use strict;
use Getopt::Long;
use XML::Simple;

# the var to store app setting and port 
my %app_setting=();
my $RUNTIME_DIR="/opt/ibm/director/vmi/data/networksetting/firewall/";

sub get_default_fw_setting
{
    # read the default/appends settings and merge them into one hash
    foreach my $f ('/opt/ibm/director/vmi/data/conf/default_firewall.xml', glob "/opt/ibm/director/vmi/data/conf/appends/firewall/*.xml"){
        my $xml=XMLin($f, ForceArray => 1, KeyAttr=>"app_id");
        @app_setting{keys %{$xml->{'net_app'}}} = values %{$xml->{'net_app'}};
    }
}

# return 1 indicates the app name is invalid
# return 0 indicates that the modification is successful
sub modify_fw_setting
{
    my ($iface, $app, $new_setting)=@_;
    if (! exists $app_setting{$app}) {
        $app=~ s/.name$//;
    }
    if (! exists $app_setting{$app}) {
        $app.=".name";
    }
    if (! exists $app_setting{$app}) {
        return 1;
    }
    $app_setting{$app}->{'default'}="deny";
    if ($new_setting eq "allowed") {
        $app_setting{$app}->{'default'}="allow";
    }
    # recreate the runtime setting file for this interface
    &create_runtime_file ($iface);

    # modify iptables restor file for this interface
    foreach my $f ("/etc/vmiiptables", "/etc/vmiip6tables") {
        my $bak=$f.".bak";
        rename $f, $bak;
        if (! open IPTABLES_BAK, "< $bak") {
            die "AE: cannot open iptables setting file for read.\n";
        }
        my $new_rules="";
        # read rules not for this interface
        while (my $line = <IPTABLES_BAK>){
            if (! ($line =~ /-A $iface-FIREWALL-SETTING/) ){
                $new_rules.=$line;
            }
            else{
                last;
            }
        }
        # append the new rules for this interface
        while (my ($k, $v) = each %app_setting) {
            # only add the rules to ACCEPT
            if ($v->{'default'} eq 'allow') {
                my $port_option="--dport";
                # handle the case of ping and pingv6
                if ($k eq "ping.name" ){
                    if ($f eq "/etc/vmiip6tables") {
                        next;
                    }
                    else {
                        $port_option="--icmp-type";
                    }
                }
                if ($k eq "pingv6.name" ){
                    if ($f eq "/etc/vmiiptables") {
                        next;
                    }
                    else {
                        $port_option="--icmpv6-type";
                    }
                }
                foreach my $port (@{$v->{'port'}}){
                    $new_rules.="-A $iface-FIREWALL-SETTING -i $iface -p $port->{'type'} -m $port->{'type'} $port_option $port->{'number'} -j ACCEPT\n";
                }
            }
        }
        # continue rules for other interface
        while (my $line =<IPTABLES_BAK>) {
            if (! ($line =~ /-A $iface-FIREWALL-SETTING/) ){
                $new_rules.= $line;
            }
        }
        close IPTABLES_BAK;

        # write new rules back
        if (! open IPTABLES, "> $f") {
            die "AE: cannot open iptables setting file for write.\n";
        }
        print IPTABLES $new_rules;
        close IPTABLES;
        unlink $bak;
    }

    return 0;
}

# to create runtime file for one interface
sub create_runtime_file 
{
    my ($iface)=@_;
    my $new_setting="";
    while(my ($k, $v) = each %app_setting){
        my $allow=0;
        if($v->{'default'} eq 'allow') {
            $allow=1;
        }
	$new_setting.="$k|$allow||\n";
    }
    if (! open RUNTIME, "> $RUNTIME_DIR/$iface-FIREWALL-SETTING"){
        die "AE: cannot initialize the firewall setting files!\n";
    }
    print RUNTIME $new_setting;
    close RUNTIME;
}

sub create_fw_setting_files 
{
    `mkdir -p $RUNTIME_DIR`;
    foreach my $iface (qw/eth0 eth1 eth2 eth3/){
        # generate the runtime files by default/appends settings
        create_runtime_file $iface;
    }

    # generate the iptables restore file
    foreach my $fiptables("/etc/vmiiptables", "/etc/vmiip6tables") {
        my $rules="*filter
:INPUT ACCEPT [0:0]
:FORWARD ACCEPT [0:0]
:OUTPUT ACCEPT [0:0]
:eth0-FIREWALL-SETTING - [0:0]
:eth1-FIREWALL-SETTING - [0:0]
:eth2-FIREWALL-SETTING - [0:0]
:eth3-FIREWALL-SETTING - [0:0]
-A INPUT -i eth0 -j eth0-FIREWALL-SETTING
-A INPUT -i eth1 -j eth1-FIREWALL-SETTING
-A INPUT -i eth2 -j eth2-FIREWALL-SETTING
-A INPUT -i eth3 -j eth3-FIREWALL-SETTING
";
        foreach my $iface (qw/eth0 eth1 eth2 eth3/){
            while (my ($k, $v) = each %app_setting) {
                # only add the rules to ACCEPT
                if ($v->{'default'} eq 'allow') {
                    # handle the case of ping and pingv6
                    my $port_option="--dport";
                    # handle the case of ping and pingv6
                    if ($k eq "ping.name" ){
                        if ($fiptables eq "/etc/vmiip6tables") {
                            next;
                        }
                        else {
                            $port_option="--icmp-type";
                        }
                    }
                    if ($k eq "pingv6.name" ){
                        if ($fiptables eq "/etc/vmiiptables") {
                            next;
                        }
                        else {
                            $port_option="--icmpv6-type";
                        }
                    }
                    foreach my $port (@{$v->{'port'}}){
                        $rules.="-A $iface-FIREWALL-SETTING -i $iface -p $port->{'type'} -m $port->{'type'} $port_option $port->{'number'} -j ACCEPT\n";
                    }
                }
            }
        }
        $rules.="COMMIT\n\n";
        if(! open IPTABLES, "> $fiptables") {
            die "AE: cannot open $fiptables to write firewall rules!\n";
        }
        print IPTABLES $rules;
        close IPTABLES;
    }
}

# the script is only called by AE
if (@ARGV %2 != 0) {
    exit 1;
}


print "AE: Configure the firewall...\n";
my %options=@ARGV;

my $iface='';
while ( my ($k, $v) = each %options){
    if ($k eq '-interface'|| $k eq '--interface') {
        $iface=$v;
        delete $options{$k};
    }
    elsif ($v ne 'allowed' && $v ne 'notallowed'){
        my $app=$k;
        $app=~ s/^-*//;
        print "AE: Invalid setting for application \'$app\'. Only 'allowed' and 'notallowed' can be specified. Ignore it.\n";
        delete $options{$k};
    }
}

if ($iface eq '') {
    print "AE: interface not specified. Ignore the configuration!\n";
    exit 1;
}

# get the default setting first
get_default_fw_setting;

if (! -e "$RUNTIME_DIR/eth0-FIREWALL-SETTING") {
    # runtime fw setting files not exist, initialized them first
    create_fw_setting_files;
}

while (my ($app, $v) = each %options) {
    $app=~ s/^-*//;
    print "AE: $v the application: $app\n";
    my $rc=modify_fw_setting ($iface, $app, $v);
    if ($rc != 0){
        print "AE: the application name $app is invalid. Ignore the configuration.\n";
    }
}
print "AE: Configure firewall completed.\n";
exit 0;

