# Licensed Materials - Property of IBM
# 
# 09N6996
#
# (C) Copyright IBM Corp. 2009, 2011 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.

SMHA_PAUSE_FILE=$SMHA_CONFIGDIR/.pause
DRBD_UPGRADE_FLAG=$SMHA_CONFIGDIR/.drbd.upgrade
MD_FILE=$SMHA_CONFIGDIR/drbd_md.conf


crt_drbd_base_rsrc () {
    
    debug "Create the DRBD base resource"
    mkrsrc IBM.Application \
    Name="$DRBDB_RSRC_NAME" \
    StartCommand="$SMHA_SCRIPTS/$DRBD_MONITOR_SCRIPT start base" \
    StopCommand="$SMHA_SCRIPTS/$DRBD_MONITOR_SCRIPT stop base" \
    MonitorCommand="$SMHA_SCRIPTS/$DRBD_MONITOR_SCRIPT status base" \
    MonitorCommandPeriod=$DRBD_MON_CMD_PERIOD \
    MonitorCommandTimeout=$DRBD_MON_CMD_TIMEOUT \
    NodeNameList="{'$HOST','$node'}" \
    StartCommandTimeout=$DRBD_MON_START_TIMEOUT \
    StopCommandTimeout=$DRBD_MON_STOP_TIMEOUT \
    UserName="root" \
    ResourceType=2

#    CleanupNodeList="{'$HOST','$node'}" \
#    CleanupCommand="$SMHA_SCRIPTS/$DRBD_MONITOR_SCRIPT cleanup base" \
#    CleanupCommandTimeout=$CLEANUP_COMMAND_TIMEOUT \

    add_to_rg $DRBDB_RSRC_NAME IBM.Application $DRBD_RG

}

crt_drbd_rsrc () {
	local name=${1}
	local device=${2}
	local mountpt=${3}
	local drbddev=${4} 
	
    mkrsrc IBM.Application \
    Name="$DRBDP_RSRC_NAME-$name" \
    StartCommand="$SMHA_SCRIPTS/$DRBD_MONITOR_SCRIPT start primary $name $device $mountpt $drbddev" \
    StopCommand="$SMHA_SCRIPTS/$DRBD_MONITOR_SCRIPT stop primary $name $device $mountpt $drbddev" \
    MonitorCommand="$SMHA_SCRIPTS/$DRBD_MONITOR_SCRIPT status primary $name $device $mountpt $drbddev" \
    MonitorCommandPeriod=$DRBD_MON_CMD_PERIOD \
    MonitorCommandTimeout=$DRBD_MON_CMD_TIMEOUT \
    NodeNameList="{'$HOST'}" \
    StartCommandTimeout=$DRBD_MON_START_TIMEOUT \
    StopCommandTimeout=$DRBD_MON_STOP_TIMEOUT \
    UserName="root" \
    ProtectionMode=1 \
    ResourceType=1
#    NodeNameList="{'$HOST','$node'}" \

#    CleanupNodeList="{'$HOST','$node'}" \
#    CleanupCommand="$SMHA_SCRIPTS/$DRBD_MONITOR_SCRIPT cleanup primary $name $device $mountpt $drbddev" \
#    CleanupCommandTimeout=$CLEANUP_COMMAND_TIMEOUT \

            
    mkrsrc IBM.Application \
    Name="$DRBDM_RSRC_NAME-$name" \
    StartCommand="$SMHA_SCRIPTS/$DRBD_MONITOR_SCRIPT start mounted $name $device $mountpt $drbddev" \
    StopCommand="$SMHA_SCRIPTS/$DRBD_MONITOR_SCRIPT stop mounted $name $device $mountpt $drbddev" \
    MonitorCommand="$SMHA_SCRIPTS/$DRBD_MONITOR_SCRIPT status mounted $name $device $mountpt $drbddev" \
    MonitorCommandPeriod=$DRBD_MON_CMD_PERIOD \
    MonitorCommandTimeout=$DRBD_MON_CMD_TIMEOUT \
    NodeNameList="{'$HOST','$node'}" \
    StartCommandTimeout=$DRBD_MON_START_TIMEOUT \
    StopCommandTimeout=$DRBD_MON_STOP_TIMEOUT \
    UserName="root" \
    ResourceType=1
    ####RunCommandsSync=0 \

#    CleanupNodeList="{'$HOST','$node'}" \
#    CleanupCommand="$SMHA_SCRIPTS/$DRBD_MONITOR_SCRIPT cleanup mounted $name $device $mountpt $drbddev" \
#    CleanupCommandTimeout=$CLEANUP_COMMAND_TIMEOUT \
    
    
    if [ $DRBD_EQU -eq 1 ]; then    
	    for h in $HOST $node
	    do
	        mkrsrc IBM.Application \
	        Name="$DRBDS_RSRC_NAME-$name" \
	        StartCommand="$SMHA_SCRIPTS/$DRBD_MONITOR_SCRIPT unused" \
	        StopCommand="$SMHA_SCRIPTS/$DRBD_MONITOR_SCRIPT unused" \
	        MonitorCommand="$SMHA_SCRIPTS/$DRBD_MONITOR_SCRIPT status sync $name $device $mountpt $drbddev" \
	        MonitorCommandPeriod=10 \
	        MonitorCommandTimeout=20 \
	        NodeNameList="{'$h'}" \
	        StartCommandTimeout=$DRBD_MON_START_TIMEOUT \
	        StopCommandTimeout=$DRBD_MON_STOP_TIMEOUT \
	        UserName="root" \
	        ResourceType=0
	    done
	    mkequ -D "Name like '${DRBDS_RSRC_NAME}-${name}'" -p A,NoControl $DRBDS_RSRC_NAME-$name-equ IBM.Application    
    fi
    
    
    debug "Adding drbd resources to storage rg"    
    add_to_rg $DRBDP_RSRC_NAME-$name IBM.Application $STORAGE_RG $ADDRGMBR_ORDER
    add_to_rg $DRBDM_RSRC_NAME-$name IBM.Application $STORAGE_RG $ADDRGMBR_ORDER

    if [ $DRBD_EQU -eq 0 ]; then
    	crt_drbd_dep_orig $DRBDP_RSRC_NAME-$name $DRBDM_RSRC_NAME-$name
    else
        crt_drbd_dep_equ $DRBDP_RSRC_NAME-$name $DRBDM_RSRC_NAME-$name $DRBDS_RSRC_NAME-$name-equ
    fi

}


crt_all_drbd_rsrcs () {
	
    # Only create the resource if it doesn't already exist
    debug "entered crt_all_drbd_rsrcs"
    lsrg -A d -g $STORAGE_RG &>/dev/null
    if [ $? -eq 6 ]; then
        
        # create the resource group
        debug "Creating storage rg"
        mkrg -p $STORAGE_RG_PRIORITY $STORAGE_RG
        mkrg -l none -p $DRBD_RG_PRIORITY $DRBD_RG 
        
        # Make sure these are not online
        take_rg_offline_n_wait $STORAGE_RG
        take_rg_offline_n_wait $DRBD_RG

        # Create the DRBD base resource
        crt_drbd_base_rsrc
        
        # Get the list of mirrors        
        local tmpf=`mktemp /tmp/smha_cadr.XXXXXX`  || fatal_error "Could not make temp file"
        $GETMIRROR_CMD | sort -k2,2 > $tmpf
        local mdigit=0
        local mdev=
        local mmir=
        local mlab=
        while read -r line
        do
        	mdev=`echo $line | cut -d' ' -f1` 
            mmir=`echo $line | cut -d' ' -f2`
            mlab=${mmir:1}
            debug "invoking crt_drbd_rsrc $mlab $mdev $mmir /dev/drbd${mdigit}" 
            crt_drbd_rsrc $mlab $mdev $mmir /dev/drbd${mdigit}
#            crt_drbd_rsrc mirror0 /dev/hda5 /mirror0 /dev/drbd0
            touch $SMHA_CONFIGDIR/${DRBD_INVALIDATE_FILE}.${mlab}
            let mdigit=mdigit+1
        done < $tmpf    
        rm $tmpf
        
        # now that everything is created, run through the mirrors again.
        # We are looking for partition mount ordering dependencies, if we find them
        # (e.g. updateslib dependent on mirror0 being mounted) we will 
        # create relationships to model them in TSA
      	local dependency=
      	tmpf=`mktemp /tmp/smha_cadr.XXXXXX`  || fatal_error "Could not make temp file"
        $GETMIRROR_CMD > $tmpf
        while read -r line
        do
        	mdev=`echo $line | cut -d' ' -f1` 
            mmir=`echo $line | cut -d' ' -f2`
            mlab=${mmir:1}
            dependency=`$GETMIRROR_DEPENDENCY_CMD -p $mmir`
            local dependent=$?
            if [ -z $dependency ]; then
            	debug "No dependency found for mount point $mmir"
            else 
            	debug "$GETMIRROR_DEPENDENCY_CMD -p $mmir returned: $dependent : $dependency"
            	local dlab=${dependency:1}
            	debug "Establishing dependsOn relationship of $mmir on $dependency"
            	make_relationship DependsOn IBM.Application:${DRBDM_RSRC_NAME}-${mlab} \
            		IBM.Application:${DRBDM_RSRC_NAME}-${dlab} ${mlab}-depon-${dlab}
            fi
        done < $tmpf
        rm $tmpf
        
        # Set the invalidate flag so we push the disk contents to the other side
        #touch $SMHA_CONFIGDIR/$DRBD_INVALIDATE_FILE
        
    else
        debug "DRBD resources already created"   
    fi	
	
}


crt_drbd_dep_orig () {
	local base=$DRBDB_RSRC_NAME
	local primary=${1}
	local mounted=${2}

    # DRBD Primary depends on DRBD base
    make_relationship DependsOn IBM.Application:${primary} IBM.Application:${base} ${primary}-depon-${base}

    # DRBD mounted depends on DRBD primary
    make_relationship DependsOn IBM.Application:${mounted} IBM.Application:${primary} ${mounted}-depon-${primary}    
        
}

crt_drbd_dep_equ () {
    local base=$DRBDB_RSRC_NAME
    local primary=${1}
    local mounted=${2}
    local syncequ=${3}

    # DRBD Primary depends on DRBD base
#    make_relationship DependsOn IBM.Application:${primary} IBM.Application:${base} ${primary}-depon-${base}
    # DRBD Primary depends on DRBD sync equ
    make_relationship DependsOn IBM.Application:${primary} IBM.Equivalency:${syncequ} ${primary}-depon-${syncequ}

    # DRBD mounted depends on DRBD primary
    make_relationship DependsOn IBM.Application:${mounted} IBM.Application:${primary} ${mounted}-depon-${primary}    
        
}

add_drbd_secondary_node () {
	 chrsrc -s "Name like '${DRBDP_RSRC_NAME-}%' && ResourceType=1" IBM.Application NodeNameList="{'$HOST','$node'}"
}

dlt_drbd_rsrc () {
    
    is_rg_created $STORAGE_RG
    if [ $? -eq 1 ]; then
        take_rg_offline_n_wait_nonfatal $STORAGE_RG
        take_rg_offline_n_wait_nonfatal $DRBD_RG
        
        rmrsrc -s "Name like '$DRBD_RSRC_NAME_SEARCH'" IBM.Application Force=1
        rmrg -s "Name=='$STORAGE_RG'"
    fi
    
}

# Mount a single mirror drive, and all bind mounts
mount_mirror_bind_mounts () {
    local mirror=${1}
    debug "mount_mirror_bind_mounts: beginning processing for $mirror"
    local mounts=`grep $mirror /etc/fstab | sed 's/ /\t/g' | grep bind | cut -f1 | grep $mirror`

    for m in ${mounts}
    do
        local spot=`grep $m /etc/fstab | grep bind | sed 's/ /\t/g' | cut -f2`
        local mount_err
        is_mounted $spot
        if [ $? -eq 0 ]; then
        	debug "Mounting $m"        
            mount_err=`LC_ALL=C mount $m`
            debug "output for mount $m command:$mount_err"
        else
        	debug "mount_mirror_bind_mounts: is_mounted returned non-zero for $spot"    
        fi
    done
}

# Mount all mirror drives
mount_mirrors () {

    # Get the list of mirrors        
    local tmpf=`mktemp /tmp/smha_mm.XXXXXX`  || fatal_error "Could not make temp file"
    $GETMIRROR_CMD > $tmpf
    while read -r line
    do
    	local mdev=`echo $line | cut -d' ' -f1` 
        local mmir=`echo $line | cut -d' ' -f2`
        local mlab=${mmir:1}

        local mount_err
        is_mounted $mmir
        if [ $? -eq 0 ]; then
            debug "Mounting mirror device $mdev on $mmir"       	
            mount_err=`LC_ALL=C mount -o rw $mdev $mmir 2>&1`
            local mount_rc=$?
            debug "output for mount $mdev $mmir rc:$mount_rc output:$mount_err"
        else
        	debug "mount_mirrors: is_mounted returned non-zero for $mmir"            
        fi

        # For each mirror, mount it.
        mount_mirror_bind_mounts $mmir
                 
    done < $tmpf    
    rm $tmpf

}

is_syncing () {
    local mirror=${1}
    local syncing=`LC_ALL=C drbdadm state ${mirror} | grep sync_progress`
    
    if [ -n "$syncing" ]; then
        return 1
    else
        return 0
    fi
    
}

any_syncing () {
    is_syncing all
    return $?
}

wait_while_syncing () {

    local forever=1
    local count=0
    local rmdr=0
    while [ $forever -gt 0 ]; do
        any_syncing
        if [ $? -eq 0 ]; then
            return
        else
            let "rmdr = count % 6"
            if [ $rmdr -eq 0 ]; then
                debug "Waiting while drbd is syncing"
            fi
            sleep 10
            let count++
        fi
    done

}

pause_mirror_sync () {
	local reason=${1}
	
	touch ${SMHA_PAUSE_FILE}-${reason}
    /sbin/drbdadm pause-sync all
}
resume_mirror_sync () {
	local reason=${1}

    rm -f ${SMHA_PAUSE_FILE}-${reason}
    ls -A ${SMHA_PAUSE_FILE}-* 2> /dev/null
    local lsout_rc=$?
    if [ $lsout_rc -eq 2 ]; then  
        /sbin/drbdadm resume-sync all
    else
        debug "Could not resume mirroring because pause files exist: rc:$lsout_rc"
    fi
}

