# Licensed Materials - Property of IBM
# 
# 09N6996 
#
# (C) Copyright IBM Corp. 2009, 2011 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.


crt_network_quorum () {
    local net_addr=${1}
    
    debug "Network quorum address:$net_addr"

    local six=
    is_address_ipv6 $net_addr    
    if [ $? -eq 1 ]; then
    	debug "Network quorum is IPv6 address:$net_addr"
    	six=6
    fi

    mkrsrc IBM.TieBreaker Type="EXEC" Name="$NET_QUORUM" \
    DeviceInfo="PATHNAME=/usr/sbin/rsct/bin/samtb_net${six} Address=$net_addr Log=1" \
    PostReserveWaitTime=30;
    
    chrsrc -c IBM.PeerNode OpQuorumTieBreaker="$NET_QUORUM"
    
}


crt_yes_quorum () {

    debug "Create Yes quorum"
        
    mkrsrc IBM.TieBreaker Type="EXEC" Name="$Y_QUORUM" \
    DeviceInfo="PATHNAME=$SMHA_SCRIPTS/tsa/smhayquorum.sh" \
    PostReserveWaitTime=30;
    
    chrsrc -c IBM.PeerNode OpQuorumTieBreaker="$Y_QUORUM"
    
}

crt_ntp_rsrc () {
    # Only create the resource if it doesn't already exist
    lsrg -A d -g $NTP_RG &>/dev/null
    if [ $? -eq 6 ]; then
        debug "Create the ntp resources"
        mkrsrc IBM.Application \
        Name="$NTP_RSRC_NAME" \
        StartCommand="$SMHA_SCRIPTS/$NTP_MONITOR_SCRIPT start" \
        StopCommand="$SMHA_SCRIPTS/$NTP_MONITOR_SCRIPT stop" \
        MonitorCommand="$SMHA_SCRIPTS/$NTP_MONITOR_SCRIPT status" \
        MonitorCommandPeriod=$NTP_MON_CMD_PERIOD \
        MonitorCommandTimeout=$NTP_MON_CMD_TIMEOUT \
        NodeNameList="{'$HOST','$node'}" \
        StartCommandTimeout=$NTP_MON_START_TIMEOUT \
        StopCommandTimeout=$NTP_MON_STOP_TIMEOUT \
        UserName="root" \
        ResourceType=2
        
        # create the resource group
        mkrg $NTP_RG
        chrg_l_none $NTP_RG
        add_to_rg $NTP_RSRC_NAME IBM.Application $NTP_RG 
        sleep 5
        #chrg -o online $NTP_RG
        debug "tsa call hook 11"
        local tsa_return
        tsa_return=$(call_tsa_chrg_o_l_u -o online $NTP_RG)
    else
        debug "ntp resources already created"   
    fi  
}

remove_ntp_autostart () {

    debug "Remove NTP autostart"

    /sbin/chkconfig -d ntp &>/dev/null
}

crt_net_alias () {

    # For each network alias, create it.
    local i
    for i in {1..5}
    do
        # See if there is an alias variable with this number
        local net_alias=network_alias__${i}
        # Dereference the constructed variable name
        local net_alias_value=${!net_alias}
        if [ -n "$net_alias_value" ]; then
            debug "Processing net alias:${net_alias_value}"
            parse_network_alias $net_alias_value
  
            # Create the equivalency between nics
            lsequ -e ${ALIAS_NIC}-equ &>/dev/null
            if [ $? -eq 6 ]; then
                debug "Make the equivalence for ${ALIAS_NIC}-equ"
                mkequ -D "Name like '${ALIAS_NIC}' && IPVersion=4" ${ALIAS_NIC}-equ IBM.NetworkInterface
            fi
            
            debug "Host: $HOST and node: $node"
            # Create the floating IP
            debug "Make the IBM.ServiceIP:${ALIAS_RSRC_NAME}${i}IP"    
            mkrsrc IBM.ServiceIP \
            Name="${ALIAS_RSRC_NAME}${i}IP" \
            NetMask=$ALIAS_MASK \
            IPAddress=$ALIAS_ADDR \
            NodeNameList="{'$HOST','$node'}"
            
            # Add this service IP to the Director RG
            add_to_rg  ${ALIAS_RSRC_NAME}${i}IP IBM.ServiceIP $DIRECTOR_RG $ADDRGMBR_ORDER

            # The alias depends on the network equivalence
            debug "Make the relationship between ServiceIP and Equivalence:${ALIAS_RSRC_NAME}${i}"            
            make_relationship DependsOn IBM.ServiceIP:${ALIAS_RSRC_NAME}${i}IP \
            IBM.Equivalency:${ALIAS_NIC}-equ \
            ${ALIAS_RSRC_NAME}${i}IP-depon-${ALIAS_NIC}-equ        
        
            # Director depends on or is collocated with this network alias
            local relationship="DependsOn"
            # If this address is not the agent manager address then it's just collocated
            if [ "$agent_manager_address" != "$ALIAS_ADDR" ]; then
                relationship="Collocated"
            fi
            
            debug "Make the relationship between Director and ServiceIP:${ALIAS_RSRC_NAME}${i}"
            make_relationship $relationship IBM.Application:$DIRECTOR_RSRC_NAME \
            IBM.ServiceIP:${ALIAS_RSRC_NAME}${i}IP \
            $DIRECTOR_RSRC_NAME-depon-${ALIAS_RSRC_NAME}${i}
            
            debug "Done processing net alias:${net_alias_value}"
            
        else
          # No more alias were found
          return                    
        fi
    
    done
        
} 


enable_ipv6 () {


	turn_on_ipv6
 
       # Turn off heartbeats for IPv6 link-local addresses
    turn_off_ipv6_linklocal_heartbeat
}

crt_net_alias6 () {

    # For each network alias, create it.
    local i
    for i in {1..5}
    do
        # See if there is an alias variable with this number
        local net_alias=network_alias6__${i}
        # Dereference the constructed variable name
        local net_alias_value=${!net_alias}
        if [ -n "$net_alias_value" ]; then
            debug "Processing net alias6:${net_alias_value}"
            parse_network_alias6 $net_alias_value

            # Create the equivalency between nics
            lsequ -e ${ALIAS_NIC}v6-equ &>/dev/null
            if [ $? -eq 6 ]; then            
                debug "Make the equivalence:${ALIAS_NIC}v6-equ"
                mkequ -D "Name like '${ALIAS_NIC}' && IPVersion=6" ${ALIAS_NIC}v6-equ IBM.NetworkInterface
            fi
            
            debug "Host: $HOST and node: $node"
            # Create the floating IP
            debug "Make the IBM.ServiceIP:${ALIAS6_RSRC_NAME}${i}IP"
            # The network prefix is optional...see if it's specified.
            if [ -n "$ALIAS_PREFIX" ]; then
                mkrsrc IBM.ServiceIP \
                Name="${ALIAS6_RSRC_NAME}${i}IP" \
                NetPrefix=$ALIAS_PREFIX \
                IPAddress=$ALIAS_ADDR \
                NodeNameList="{'$HOST','$node'}"
            else
                mkrsrc IBM.ServiceIP \
                Name="${ALIAS6_RSRC_NAME}${i}IP" \
                IPAddress=$ALIAS_ADDR \
                NodeNameList="{'$HOST','$node'}"
            fi
            
            # Add this service IP to the Director RG
            add_to_rg ${ALIAS6_RSRC_NAME}${i}IP IBM.ServiceIP $DIRECTOR_RG $ADDRGMBR_ORDER

            # The alias depends on the network equivalence
            debug "Make the relationship between ServiceIP and Equivalence:${ALIAS6_RSRC_NAME}${i}"
            make_relationship DependsOn IBM.ServiceIP:${ALIAS6_RSRC_NAME}${i}IP \
            IBM.Equivalency:${ALIAS_NIC}v6-equ \
            ${ALIAS6_RSRC_NAME}${i}IP-depon-${ALIAS_NIC}v6-equ        
        
            # Director depends on or is collocated with this network alias
            local relationship="DependsOn"
            # If this address is not the agent manager address then it's just collocated
            if [ "$agent_manager_address" != "$ALIAS_ADDR" ]; then
                relationship="Collocated"
            fi
            
            debug "Make the relationship between Director and ServiceIP:${ALIAS6_RSRC_NAME}${i}"
            make_relationship $relationship IBM.Application:$DIRECTOR_RSRC_NAME \
            IBM.ServiceIP:${ALIAS6_RSRC_NAME}${i}IP \
            $DIRECTOR_RSRC_NAME-depon-${ALIAS6_RSRC_NAME}${i}
            
            debug "Done processing net alias:${net_alias_value}"
            
        else
          # No more alias were found
          return                    
        fi
    
    done
        
} 

# Mark all RSCT network resources to have IPV6 enabled
turn_on_ipv6 () {
	# first off, let's loop until we see any network resources, so we know RSCT is far enough along to issue a change
	while true; do
		# there should always be at least an IPv6 link local.
		# wait until we get ANYTHING back
		LC_ALL=C lsrsrc IBM.NetworkInterface
		rc=$?
		case $rc in
        0)
        	# found something.  Go ahead and issue the change.
			chrsrc_out=`chrsrc -c IBM.NetworkInterface IPv6Support=1`
			rc=$?
			# kick off a refresh of the IBM.NetworkInterface resources
        	refrsrc IBM.NetworkInterface
        	# This is supposed to complete within about 30 seconds
        	sleep 60  
			break 
        ;;
        6)
        	# No resource found - sleep a bit
			debug "turn_on_ipv6: sleeping waiting for IBM.NetworkInterface IPV6 resources"
			sleep 10
		;;
        *)
           	debug "turn_on_ipv6: giving up. lsrsrc returned $rc"
			break
        ;;
        esac
      
	done
		
	debug "turn_on_ipv6: rc=$rc : output is: $chrsrc_out"
}

# RSCT does not handle link local properly for sending heartbeats,
# so we need to be sure that we do not have RSCT sending a heartbeat
# on an IPv6 link-local address.  The link-local addresses are created
# automatically and I don't believe that they can ever change.  
turn_off_ipv6_linklocal_heartbeat() {
	
	# first off, let's loop until we see IPV6 resources, so we know RSCT is far enough along to issue a change
	while true; do
		# there should always be at least an IPv6 link local.
		# wait until we get ANYTHING back
		LC_ALL=C lsrsrc -s 'IPVersion == 6' IBM.NetworkInterface
		rc=$?
		case $rc in
        0)
        	# found something.  Go ahead and issue the change.
			chrsrc_out=`chrsrc -s 'IPVersion == 6 && (IPAddress LIKE "fe80%" || IPAddress LIKE "FE80%")' IBM.NetworkInterface HeartbeatActive=0 2>/dev/null`
			rc=$?
			break 
        ;;
        6)
        	# No resource found - sleep a bit
			debug "turn_off_ipv6_linklocal_heartbeat: sleeping waiting for IBM.NetworkInterface IPV6 resources"
			sleep 10
		;;
        *)
           	debug "turn_off_ipv6_linklocal_heartbeat: giving up. lsrsrc returned $rc"
			break
        ;;
        esac
      
	done
		
	debug "turn_off_linklocal_heartbeat: rc=$rc : output is: $chrsrc_out"
}

# We want RSCT to always send heartbeats on global IPv6 addresses.
# If an interface only has link-local, but later has an IPv6 global 
# added, we want to pick that up.  So, this should be run every time
# that a reboot is done (chnetcfg requires a reboot).  Both of these
# heartbeat routines are "hacks" to work around an RSCT issue with how
# the socket address structure is handled for link-local addresses.  We 
# can likely remove these once we get to the 6.3 timeframe.
turn_on_ipv6_global_heartbeat() {
	# first off, let's loop until we see IPV6 resources, so we know RSCT is far enough along to issue a change
	while true; do
		# there should always be at least an IPv6 link local.
		# wait until we get ANYTHING back
		LC_ALL=C lsrsrc -s 'IPVersion == 6' IBM.NetworkInterface
		rc=$?
		case $rc in
        0)
        	# found something.  Go ahead and issue the change.
        	# If something bad has happened such that we don't have configuration quorum now,
        	# this routine will wait for up to 2 hours in the hope that the peer node comes back online
			loop_set_heartbeat_on
			rc=$?
			break 
        ;;
        6)
        	# No resource found - sleep a bit
			debug "turn_on_ipv6_global_heartbeat: sleeping waiting for IBM.NetworkInterface IPV6 resources"
			sleep 10
		;;
        *)
           	debug "turn_on_ipv6_global_heartbeat: giving up. lsrsrc returned $rc"
			break
        ;;
        esac
	
	done
	debug "turn_on_ipv6_global_heartbeat: rc=$rc"
	return $rc
}

# Try to handle case where we don't have configuration quorum.
loop_set_heartbeat_on () {
	for i in {1..240}
    do
        debug "loop_on_set_heartbeat_on: attempting to set global IPv6 heartbeating"
        # found something.  Go ahead and issue the change.
		chrsrc_out=`chrsrc -s 'IPVersion == 6 && !(IPAddress LIKE "fe80%" || IPAddress LIKE "FE80%")' IBM.NetworkInterface HeartbeatActive=1 2>/dev/null`
		rc=$?
        case $rc in
        0)
            break;
        ;;
        6) 
        	# 6 indicates that no global resources found.  Reset to 0 and return.
        	debug "loop_on_set_heartbeat_on: chrsrc found no global IPv6 resources: rc=6"
        	rc=0
        	break;
        ;;
        *)
            sleep 30
        ;;
        esac
    done
    debug "loop_on_set_heartbeat_on: chrsrc_out: ${chrsrc_out}   :   rc=${rc}"
	return $rc
}

crt_network_aliases () {
 	# enable IPv6 in RSCT - and set heart-beating appropriately
    enable_ipv6
    crt_net_alias
    crt_net_alias6
    # crt_arp_rsrc
}


