#!/bin/bash
# Licensed Materials - Property of IBM
# 
# 09N6996
#
# (C) Copyright IBM Corp. 2009, 2010 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.

PROGNAME=$(basename $0)
PROGPATH=$(dirname $0)
EXTPROGNAME="smhaVRMs"

INSTALL_DIR=/opt/ibm/director
CMD_DIR=$INSTALL_DIR/ha/bin/priv
COMMON_CODE=/opt/ibm/director/ha/bin/include/smhacommon.sh

if [ -f $COMMON_CODE ]; then
    source $COMMON_CODE
else
    echo "$PROGNAME:Common code not found."
    exit 1
fi

load_common_code $INSTALL_DIR/ha/bin/include/smhaparms.sh

PARM_FLAGS="g get r rsct t tsa"
PARM_MULTI=""
PARM_VALID=" g get r rsct t tsa"

###################################
## Functions                     ##
###################################



#############################################################
# Returns the Installed Version Number ($IVN) and the 
# Active Version Number ($AVN) of the TSA version, as output 
# by the lssrc -ls IBM.RecoveryRM command.
#############################################################
get_TSA_Versions () {
	IVN=
    AVN=

	lssrc_out=`LC_ALL=C /usr/bin/lssrc -ls IBM.RecoveryRM`
	debug "output of lssrc is: $lssrc_out"
	IVN=`LC_ALL=C /usr/bin/lssrc -ls IBM.RecoveryRM | grep -e "Our IVN" | cut -d: -f2 | awk '{print $1}'`
	debug  "IVN is:  $IVN"
	AVN=`LC_ALL=C /usr/bin/lssrc -ls IBM.RecoveryRM | grep -e "Our AVN" | cut -d: -f2 | awk '{print $1}'`
	debug "AVN is:  $AVN"
}

#############################################################
# Gets the RSCT version number ($LOCAL_RSCTVN) for this node.
#############################################################
get_RSCT_Versions () {
	LOCAL_RSCTAVN=
	LOCAL_RSCTIVN=
	get_local_rsct_node_name
	
	ctversion_out=`/usr/sbin/rsct/install/bin/ctversion`
	debug "output of ctversion is: $ctversion_out"
	lsrpnode_out=`LC_ALL=C /usr/bin/lsrpnode`
	debug "output of lsrpnode is: $lsrpnode_out"
	
	LOCAL_RSCTAVN=`LC_ALL=C /usr/bin/lsrpnode -L | grep -e $LOCAL_RSCT_NODE_NAME | awk '{print $3}'`
	debug  "LOCAL_RSCTAVN is:  $LOCAL_RSCTAVN"
	
	LOCAL_RSCTIVN=`LC_ALL=C /usr/sbin/rsct/install/bin/ctversion | awk '{print $2}'`
	debug "LOCAL_RSCTIVN is: $LOCAL_RSCTIVN"
}


proc_parms () {
    
    # Parse the parameters
    # The result should be that variables are set for each parameter and
    # CMD_PARMS will contain the list of parameters processed
    # If a file is specified it will also read the file.
    parse_all_parms $*
    
    # this is only for debug
    #list_parm_vars 
    
}

#####################################################
# Process the get parameter.  Collect TSA and RSCT version info.
#####################################################
proc_get () {
    
    # if the get parameter was specified then handle it.
    if [ -n "$get" ]; then
        
        # collect all the info
        get_TSA_Versions
		echo "IVN=$IVN"
		echo "AVN=$AVN"
		get_RSCT_Versions
		echo "LOCAL_RSCTAVN=$LOCAL_RSCTAVN"
		echo "LOCAL_RSCTIVN=$LOCAL_RSCTIVN"
        RC=$?
        
        exit $RC
    fi
    
}

#####################################################
# Process the rsct parameter.
# issue runact c IBM.PeerDomain CompleteMigration Options=0
#####################################################
proc_rsct () {

   # if the get parameter was specified then handle it.
    if [ -n "$rsct" ]; then
		/usr/bin/runact -c IBM.PeerDomain CompleteMigration Options=0
		RC=$?
		
		exit $RC
	fi
}

#####################################################
# Process the tsa parameter.
# issue samctrl -m
#####################################################
proc_tsa () {

   # if the get parameter was specified then handle it.
    if [ -n "$tsa" ]; then
		/usr/bin/samctrl -m <<<Y
		RC=$?
		
		exit $RC
	fi
}

###################################
## Main code                     ##
###################################


if [ $# -eq 0 ]; then
    
    exit 0
fi

proc_parms $*

# process get - i.e. collect all VRM info
proc_get

# process rsct - i.e. issue the RSCT migration command
proc_rsct

# process tsa - i.e. issue the TSA migration command
proc_tsa

exit 0



