#!/bin/bash
# Licensed Materials - Property of IBM
# 
# 09N6996
#
# (C) Copyright IBM Corp. 2009, 2010 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.

PROGNAME=$(basename $0)
PROGPATH=$(dirname $0)

export PATH=/opt/ibm/director/bin:/opt/ibm/director/lwi/bin:/bin:/usr/bin:/usr/sbin:/sbin:/usr/smrshbin/

INSTALL_DIR=/opt/ibm/director
CMD_DIR=$INSTALL_DIR/ha/bin/priv
COMMON_CODE=/opt/ibm/director/ha/bin/include/smhacommon.sh

if [ -f $COMMON_CODE ]; then
    source $COMMON_CODE
else
    echo "$PROGNAME:Common code not found."
    exit 1
fi

#################################################
#  Return codes
#################################################
RETURN_NO_DHCP_CONF=50
RETURN_DHCP_IS_CONF=51
RETURN_SHARED_DHCP_CONF=52
RETURN_NONSHARED_DHCP_CONF=53
RETURN_CONFLICT_DHCP_CONF=54
RETURN_SECONDARY_DHCP_CONF=55

#################################################
#  Functions
#################################################

get_v4_nics () {
    
    local nics=`LC_ALL=C lsnetcfg -n | grep ipv4addr | grep -v "0.0.0.0" | grep -o -e "ipv4addr_.*:" | sed "s/ipv4addr_//g;s/://g"`
#    local nics=`lsnetcfg -n | grep ipv4addr_ | cut -d' ' -f1`
    V4_NICS=$nics
}

get_v4_addr_for_nic () {
    local nic=$1
    
    local addr=`LC_ALL=C lsnetcfg -n | grep ipv4addr_${nic} | sed "s/ //g" | cut -d':' -f2`
    V4_ADDR=$addr
}

get_netmask_for_nic () {
    local nic=${1}
    
    local mask=`LC_ALL=C lsnetcfg -n | grep "ipv4netmask_${nic}" | sed 's/ //g' | cut -d':' -f2`
    V4_NETMASK=$mask
    
}

get_network_seg () {
    local addr=$1
    local mask=$2
    
    # Loop through the addresse and mask
    local mremain=$mask
    local aremain=$addr
    local seg=
    for i in {1..4}
    do
        local mnum=${mremain##*.}
        mremain=${mremain%.*}
        local anum=${aremain##*.}
        aremain=${aremain%.*}
        
        local netnum
        if [ $mnum -ne 0 ]; then
            let "netnum = anum & mnum"
        else
            netnum=0
        fi
        
        if [ -z $seg ]; then
            seg=$netnum
        else    
            seg=${netnum}.${seg}
        fi
        
    done
    
    V4_SEG=$seg
    
}


get_dhcpd_nics () {
	#fake it out for testing
#	V4_DHCPD_NICS="eth0 eth1";return
	

    #local nics=`grep _INTERFACE /etc/sysconfig/dhcpd | grep -v ^# | cut -d'"' -f2`
    local nics=`LC_ALL=C lsnetcfg -n | grep dhcpserver_ | grep on | sed -e "s/dhcpserver_//g" -e "s/: on//g" -e "s/ //g"`
    V4_DHCPD_NICS=$nics    

}

get_all_network_info() {
    
#   get_v4_nics
#   local nics=$V4_NICS
    
    get_dhcpd_nics
    local nics=$V4_DHCPD_NICS
    
    for n in $nics
    do
        get_v4_addr_for_nic $n
        local addr=$V4_ADDR
        get_netmask_for_nic $n
        local netmask=$V4_NETMASK

        get_network_seg $addr $netmask
        
        echo "$n,$addr,$netmask"
        echo "network seg: $V4_SEG"
    done
    
}

get_network_info () {
    
    get_dhcpd_nics
    local nics=$V4_DHCPD_NICS

    local result=
    local n=    
    for n in $nics
    do
        get_v4_addr_for_nic $n
        local addr=$V4_ADDR
        get_netmask_for_nic $n
        local netmask=$V4_NETMASK
        
        debug "$n,$addr,$netmask"
        
        if [ -z $result ]; then
            result=$n,$addr,$netmask
        else
            result="$n,$addr,$netmask $result"
        fi
    done
    DHCP_NETWORK_INFO=$result    
}

check_for_local_dhcp () {
	
	get_dhcpd_nics
#	echo "check local:$V4_DHCPD_NICS"
	if [ -z "$V4_DHCPD_NICS" ]; then
		return $RETURN_NO_DHCP_CONF
	else
	    return $RETURN_DHCP_IS_CONF
	fi
}

# Format the network information to
# send to this same script on the remote
# system.
get_network_info_to_send () {
	
    get_network_info
    # output the results
    echo "\"$DHCP_NETWORK_INFO\""
}


is_nic_in_nics () {
    local nic=$1
    shift
    local nics=$*
    
    local nnic
    for nnic in $nics
    do
        echo " is_nic_in_nics -nic: $nic nnic:$nnic"
        if [[ "$nnic" == "$nic" ]]; then
            return 1
        fi        
    done
    
    return 0
}


is_nic_in_networks () {
	local nic=$1
	shift
	local networks=$*
	
    local n=
    for n in $networks
    do
        local nnic=${n%%,*}
        echo " is_nic_in_network -nic: $nic nnic:$nnic"
        if [[ "$nnic" == "$nic" ]]; then
        	return 1
        fi        
    done
	
	return 0
}

check_for_unique_secondary_dhcp () {
	local sec_nets=$1
	local prim_nets=$2
    local n=
    
    for n in $sec_nets
    do
        local nnic=${n%%,*}
        is_nic_in_networks $nnic "$prim_nets"
        if [ $? -eq 0 ]; then
            return $RETURN_SECONDARY_DHCP_CONF
        fi        
    done
    
    return 0
}

compare_networks_to_other () {
	local dhcp_model=$1
	local nets1=$2
	local nets2=$3
	
	# loop through the one network and compare against the other
    local n=
    for n in $nets1
    do
        local thenic=${n%%,*}
        
        is_nic_in_networks $thenic $nets2
        local found_nic=$?
        
        echo "compare -- network is:$n and nic is:$thenic found is:$found_nic in $nets2"
        
        # Now record what we found
        local nic_model=
        if [ $found_nic -eq 1 ]; then
            # we found it, they are in non-shared
            echo "found nic: $thenic in $nets2"
            nic_model=$RETURN_NONSHARED_DHCP_CONF
        else
            # we didn't find it, they are shared
            nic_model=$RETURN_SHARED_DHCP_CONF          
        fi
        
        #Check what we found to what we found in the past
        echo "nic: $thenic dhcp_model:$dhcp_model nic_model:$nic_model"
        if [[ "$dhcp_model" == "9999" ]]; then
            #Just record what we found
            dhcp_model=$nic_model
        else
            #Otherwise they have to match
            if [[ "$dhcp_model" != "$nic_model" ]]; then
                return $RETURN_CONFLICT_DHCP_CONF
            fi
        
        fi
                
    done
	
	return $dhcp_model
}

compare_networks () {
	# input to this function is $networks
	
	# Get the local networking info
	get_network_info
	local lnetworks=$DHCP_NETWORK_INFO
	local rnetworks=$networks
	
	# For every remote DHCP network, ensure the local nic is configured
	get_v4_nics
	local lv4nics=$V4_NICS
	
	local rnet
    for rnet in $rnetworks
    do
        local rnic=${rnet%%,*}
        is_nic_in_nics $rnic $lv4nics
        if [ $? -eq 0 ]; then
        	debug "Remote nic $rnic not configured locally. Local list: $lv4nics"
            return $RETURN_CONFLICT_DHCP_CONF
        fi        
    done

    # If there aren't any local dhcp servers just return as shared
    if [ -z "$lnetworks" ]; then
    	debug "No local dhcp servers configured. Return shared"
        return $RETURN_SHARED_DHCP_CONF
    fi

    ## Before we determine share or non-shared model,
    ## see if there are nics on this node that aren't on the other.
    ## We don't support that, since we don't copy or merge in the configs.
    check_for_unique_secondary_dhcp "$lnetworks" "$rnetworks"
    local sec_rc=$?
    if [ $sec_rc -ne 0 ]; then
    	debug "DHCP model return: $sec_rc"
    	return $sec_rc
    fi

	## If we get here, we have DHCP configured on the remote and local system
	## Loop through making sure they are either
	## all shared or all non-shared, by comparing the nics.

    # Set the dhcp model to neutral
    local dhcp_model=9999		
	
	compare_networks_to_other $dhcp_model "$lnetworks" "$rnetworks"
	dhcp_model=$?
    if [[ "$dhcp_model" != "$RETURN_CONFLICT_DHCP_CONF" ]]; then
        compare_networks_to_other $dhcp_model "$rnetworks" "$lnetworks"
        dhcp_model=$?
        
    fi
	
	debug "DHCP model return: $dhcp_model"	
	return $dhcp_model
}


#################################################
#  Main code
#################################################
RC=0
OPTERR=0
while getopts Pn:cO: flag;do
    case $flag in
        P) gatherPrimary=1;;
        n) networks=$OPTARG;;
        c) checkForDHCP=1;;
    esac
    
done


# If this is the gather call, call the function
if [ "$gatherPrimary" == "1" ]; then
	get_network_info_to_send
	exit $RC
fi

if [ -n "$networks" ]; then
	
	let "count = OPTIND - 1"
    shift $count
    
    while [ $# -gt 0 ]
    do
    	networks="$networks ${1}"
    	shift
    done
    echo "networks: $networks"        
	compare_networks
	exit $?
fi
if [ "$checkForDHCP" == "1" ]; then
    check_for_local_dhcp
    exit $?
fi


