#!/bin/bash
# Licensed Materials - Property of IBM
# 
# 09N6996
#
# (C) Copyright IBM Corp. 2009, 2010 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.

PATH=/usr/bin:$PATH
PROGNAME=$(basename $0)
PROGPATH=$(dirname $0)

INSTALL_DIR=/opt/ibm/director
CMD_DIR=$INSTALL_DIR/ha/bin/priv
COMMON_CODE=/opt/ibm/director/ha/bin/include/smhacommon.sh

if [ -f $COMMON_CODE ]; then
    source $COMMON_CODE
else
    echo "$PROGNAME:Common code not found."
    exit 1
fi
#############################################################

PARM_FLAGS="h help u users f firewall r prep d drbd t terminate T tsa a authentication-mech c update-interfaces N ntp B dns y syslog"
PARM_MULTI="network-alias network-alias6"
PARM_TRANSLATION="n node \
N ntp \
B dns \ 
p password \
u users \
U user \
C cuser \
f firewall \
D dhcp \
d drbd \
L local-replication-address \
R remote-replication-address \
i network-alias \
I network-alias6 \
q network-quorum \
g agent-manager-address \
r prep \
x node-id \
t terminate \
T tsa \
a authentication-mech \
c update-interfaces \
y syslog \
h help" 
PARM_SINGLES="help users firewall terminate authentication-mech update-interfaces dns ntp syslog"
PARM_VALID=$PARM_TRANSLATION
PARM_DONT_SAVE="help tsa password cuser"

load_common_code $INSTALL_DIR/ha/bin/include/smhaparms.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhadrbd.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhadir.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhadb2.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhamodel.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhacim.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhaplugin.sh
load_common_code $INSTALL_DIR/ha/bin/include/smharsrc.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhaarp.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhadhcp.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhaagent.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhaerrcode.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhainitcom.sh


###################################
## Functions                     ##
###################################
proc_parms () {
    
    # Parse the parameters
    # The result should be that variables are set for each parameter and
    # CMD_PARMS will contain the list of parameters processed
    # If a file is specified it will also read the file.
    parse_all_parms $*    
        
}

# requires that $HOST and $node are already set.
create_keystores() {
	
	    local LOCAL_NODE=$HOST
        local REMOTE_NODE=$node
        local KEYSTORE_PASS=ibmpassw0rd
       	
       	local LOCAL_BROKER_FILE=$BROKER_KS
       	local LOCAL_BROKER_CERT=$SMHA_CONFIGDIR/local_broker_cert
       	local REMOTE_BROKER_FILE=$SMHA_CONFIGDIR/remote_broker.ks
       	local REMOTE_BROKER_CERT=$SMHA_CONFIGDIR/remote_broker_cert
       	
       	if [ -f $LOCAL_BROKER_FILE ]; then
        	rm $LOCAL_BROKER_FILE
    	fi
    	if [ -f $LOCAL_BROKER_CERT ]; then
        	rm $LOCAL_BROKER_CERT
    	fi
    	if [ -f $REMOTE_BROKER_FILE ]; then
        	rm $REMOTE_BROKER_FILE
    	fi
    	if [ -f $REMOTE_BROKER_CERT ]; then
        	rm $REMOTE_BROKER_CERT
    	fi
    	
       	# Change the mask so that new files aren't world readable.
       	local mask=`umask`
       	umask 077
       	
       	# create local keystore and export the CA
		$JREBIN_DIR/keytool -genkeypair -alias broker -keyalg RSA -validity 9999 -dname CN=$LOCAL_NODE,O=IBM -keystore $LOCAL_BROKER_FILE -storepass $KEYSTORE_PASS -keypass $KEYSTORE_PASS
		 RC=$?
        if [ $RC -ne 0 ]; then
            return_error $SSL_KEYS_NOT_CREATED "Error creating local host self signed cert RC=$RC"
        fi
                
		$JREBIN_DIR/keytool -genkeypair -alias broker-localhost -keyalg RSA  -validity 9999 -dname CN=localhost,O=IBM -keystore $LOCAL_BROKER_FILE -storepass $KEYSTORE_PASS -keypass $KEYSTORE_PASS
		 RC=$?
        if [ $RC -ne 0 ]; then
            return_error $SSL_KEYS_NOT_CREATED "Error creating local localhost self signed cert RC=$RC"
        fi
        
		$JREBIN_DIR/keytool -export -alias broker -keystore $LOCAL_BROKER_FILE -storepass $KEYSTORE_PASS -file $LOCAL_BROKER_CERT
		 RC=$?
        if [ $RC -ne 0 ]; then
            return_error $SSL_KEYS_NOT_CREATED "Error exporting local host cert RC=$RC"
        fi
        
		# create remote keystore and export it's CA
		$JREBIN_DIR/keytool -genkeypair -alias broker -keyalg RSA  -validity 9999  -dname CN=$REMOTE_NODE,O=IBM -keystore $REMOTE_BROKER_FILE -storepass $KEYSTORE_PASS -keypass $KEYSTORE_PASS
		 RC=$?
        if [ $RC -ne 0 ]; then
            return_error $SSL_KEYS_NOT_CREATED "Error creating remote host self signed cert RC=$RC"
        fi
        
		$JREBIN_DIR/keytool -genkeypair -alias broker-localhost -keyalg RSA  -validity 9999 -dname CN=localhost,O=IBM -keystore $REMOTE_BROKER_FILE -storepass $KEYSTORE_PASS -keypass $KEYSTORE_PASS
		 RC=$?
        if [ $RC -ne 0 ]; then
            return_error $SSL_KEYS_NOT_CREATED "Error creating remote host localhost cert RC=$RC"
        fi
        
		$JREBIN_DIR/keytool -export -alias broker -keystore $REMOTE_BROKER_FILE -storepass $KEYSTORE_PASS -file $REMOTE_BROKER_CERT
	 	RC=$?
        if [ $RC -ne 0 ]; then
            return_error $SSL_KEYS_NOT_CREATED "Error exporting remote host cert RC=$RC"
        fi
        
		# import the CA from the remote into the local Keystore
		$JREBIN_DIR/keytool -import -alias client -keystore $LOCAL_BROKER_FILE -storepass $KEYSTORE_PASS -trustcacerts -file $REMOTE_BROKER_CERT <<-EOF
		yes
		EOF
		 RC=$?
        if [ $RC -ne 0 ]; then
            return_error $SSL_KEYS_NOT_CREATED "Error importing remote cert into local keystore RC=$RC"
        fi
        
		# import the CA from the local into the remote Keystore
		$JREBIN_DIR/keytool -import -alias client -keystore $REMOTE_BROKER_FILE -storepass $KEYSTORE_PASS -trustcacerts -file $LOCAL_BROKER_CERT <<-EOF
		yes
		EOF
       	 RC=$?
        if [ $RC -ne 0 ]; then
            return_error $SSL_KEYS_NOT_CREATED "Error importing local cert into remote keystore RC=$RC"
        fi
       	
       	# delete the cert files, we don't need them anymore
		rm $LOCAL_BROKER_CERT
		rm $REMOTE_BROKER_CERT
 
        umask $mask
              	
       	return $RC
       	
}


#####################################################
# Process the help parameter
#####################################################
proc_help () {
    
    if [[ "$help" == "Y" ]]; then
        echo_help
        exit 0
    fi

}

echo_help () {
        echo "${PROGNAME}: [-h][-n <node>]"
        echo ""
        echo "The ${PROGNAME} command is used to start the HA daemon."
        echo "Options:"
        echo ""
        echo "  -h,--help                       Display this help"
        echo "  -n,--node                       Specifies the companion node in the configuration"
        echo " "
}

#####################################################
# Process the node parameter
#####################################################
proc_node () {
    
    # if the node parameter was specified then handle it.
    if [ -n "$node" ]; then
    	
    	# Remove any MQ data
        rm -rf $SMHA_DIR/persist/*
    	
    	# create keystores for both nodes
        create_keystores
        
        RC=$?
        if [ $RC -ne 0 ]; then
            return_error $SSL_KEYS_NOT_CREATED "Error setting up keystores RC=$RC"
        fi
    	
    	# start the mq daemon now - takes longer
        /etc/rc.d/smhamq restart
       	
       	wait_for_mq_online
       	RC=$?
       	if [ $RC -ne 0 ]; then
            return_error $MQ_NOT_STARTING "MQ not coming online. RC=$RC"
        fi
       	       	
    	# Run the code to initialize HA
    	if [ -z "$user" ]; then
    		get_admin_id
    		user=$ADMIN_ID
    	fi
    	
    	if [ -z "$cuser" ]; then
    		cuser=$user
    	fi
    	
    	save_cfg_user_id $cuser
    	run_init "init" $node $user $password $cuser
    	RC=$?
    	    	    	
    	record_cfg_status "`eval_gettext "Synchronizing resources."`"
    	
        exit $RC
    fi
    
}


#####################################################
# Process the authentication-mech parameter
#####################################################
proc_authentication_mech () {
    
    # if the authentication-mech parameter was specified then handle it.
    if [ -n "$authentication_mech" ]; then
        
        # Run the code to sync remote authentication mechanism configuration files (kerberos and LDAP)
        run_init "authentication_mech" 
        RC=$?
        
        exit $RC
    fi
    
}



#####################################################
# Process the dns parameter
#####################################################
proc_dns () {
    
    # if the dns parameter was specified then handle it.
    if [ -n "$dns" ]; then
        
        # Run the code to sync the DNS BIND resolver file (/etc/resolv.conf) 
        run_init "dns" 
        RC=$?
        
        exit $RC
    fi
    
}

#####################################################
# Process the syslog parameter
#####################################################
proc_syslog () {
    
    # if the syslog parameter was specified then handle it.
    if [ -n "$syslog" ]; then
        
        # Run the code to sync the sylog config file (/etc/syslog.conf) 
        run_init "syslog" 
        RC=$?
        
        exit $RC
    fi
    
}

#####################################################
# Process the ntp parameter
#####################################################
proc_ntp () {
    
    # if the ntp parameter was specified then handle it.
    if [ -n "$ntp" ]; then
        
        # Run the code to sync the DNS BIND resolver file (/etc/ntp.conf) 
        run_init "ntp" 
        RC=$?
        
        exit $RC
    fi
    
}


#####################################################
# Process the users parameter
#####################################################
proc_users () {
    
    # if the users parameter was specified then handle it.
    if [ -n "$users" ]; then
        
        # Run the code to sync users
        run_init "users" 
        RC=$?
        
        exit $RC
    fi
    
}

#####################################################
# Process the firewall parameter
#####################################################
proc_firewall () {
    
    # if the firewall parameter was specified then handle it.
    if [ -n "$firewall" ]; then
        
        # Run the code to sync the firewall
        run_init "firewall" 
        RC=$?
        
        exit $RC
    fi
    
}

#####################################################
# Process the update-interfaces parameter
#####################################################
proc_update_interfaces () {
    
    # if the firewall parameter was specified then handle it.
    if [ -n "$update_interfaces" ]; then
        
        # Run the code to update just the firewall interfaces
        run_init "update-interfaces" 
        RC=$?
        
        exit $RC
    fi
    
}

#####################################################
# Process the prep parameter
#####################################################
proc_prep () {
    
    # if the prep parameter was specified then handle it.
    if [ -n "$prep" ]; then
        
        record_cfg_status "`eval_gettext "Preparing the nodes to join the cluster."`"
        # Run the code to initialize HA
        run_init "prep" $node_id
        RC=$?
        
        exit $RC
    fi
    
}

#####################################################
# Process the drbd parameter
#####################################################
proc_drbd () {
    
    # if the drbd parameter was specified then handle it.
    if [ -n "$drbd" ]; then
        
        # Run the code to initialize HA
        run_init "drbd"  $local_replication_address $remote_replication_address
        RC=$?
        
        exit $RC
    fi
    
}

#####################################################
# Process the terminate parameter
#####################################################
proc_term () {
    
    # if the terminate parameter was specified then handle it.
    if [ -n "$terminate" ]; then

        debug "Starting termination code."
            
        # Run the code to terminate HA
        record_starting_termination
        
        run_conf_plugins unconf begin
        
        # Only run the init if our mq is running, otherwise we'll
        # hang trying to send a message and HA is probably not
        # configured anyway.
        /etc/rc.d/smhamq status
        if [ $? -eq 0 ]; then        
            run_init "term"
        fi
        
        delete_tsa_model
        
        # Do any final termination code.  This is common between nodes
        $SMHA_SCRIPTS/priv/smhaterm.sh --restore
        
        RC=$?
        
        exit $RC
    fi
    
}

#####################################################
# Process the dhcp parameter
#####################################################
proc_dhcp_on_off () {
    
    # if the dhcp parameter was specified without the full TSA setup, then handle it.
    if [ -n "$dhcp" ]; then
        RC=0
        check_key_values dhcp enable disable
        
        debug "Processing dhcp command:$dhcp"
        is_domain_setup
        if [ $? -eq 1 ]; then
        	# Don't allow dhcp to be enabled/disabled unless it was previously setup
        	get_value_from_cfg dhcp
        	local share_val=$CFG_RETURN_VALUE        
            if [ -n "$share_val" ]; then

	            case $dhcp in
	            "enable" )
                    get_value_from_cfg node
    	            node=$CFG_RETURN_VALUE        
	            
	                # create the resources
	                crt_dhcp_rsrc $share_val
	                crt_dhcp_rsrc_step2
	                sleep 5
	                # chrg -o online $DHCP_RG
	                debug "tsa call hook 12"
	                local tsa_return
	                tsa_return=$(call_tsa_chrg_o_l_u -o online $DHCP_RG)
	                RC=$?
	                if [ $RC -ne 0 ]; then
	                    return_error $RC "Error making $DHCP_RG online. RC=$RC $tsa_return"
	                fi
	                
	            ;;
	            "disable" )
	                dlt_dhcp_rsrc
	            ;;
	            esac
            	        	        	
            else
                return_error $DHCP_WAS_NOT_CONFIGURED_WITH_HA "DHCP Services were not configured with HA."            
            fi
        	        	
        else
            return_error $HA_ENVIRONMENT_NOT_CONFIGURED "The HA environment is not configured."
        fi
                
        exit $RC
    fi
    
}


#####################################################
# Process the tsa parameter
#####################################################
proc_tsa () {
    
    # if the tsa parameter was specified then handle it.
    if [ -n "$tsa" ]; then
        
        record_cfg_status "`eval_gettext "Configuring local agent."`"        
        register_meps
        turn_off_address_change_processing
        
        remove_director_autostart
        save_dirstartservice_status
        remove_dirstartservice
        
        # Take down Director (Should already be done by the GUI scripts)
        debug "Stopping director."
        record_cfg_status "`eval_gettext "Stopping Director on the primary node."`"        
        $SMHA_SCRIPTS/tsa/smhatsadir.sh stop 
        
        # Comment out the DB2 FMC and turn off auto start
        comment_fmc
        db2auto_off
        add_ha_to_inittab
        
        # Create the flag file so when init q is run in the next step
        # the smhamaint.sh script doesn't do anything.  It needs to be
        # run now though, so it doesn't run later when some other 
        # script calls init q
        create_stop_maint_flagfile
        
        #Settings in /etc/inittab were changed. Poke init to read the new config
		init q
        
        # Stop db2 (Should already be done by the GUI scripts)
        debug "Stopping db2."
        record_cfg_status "`eval_gettext "Stopping database on the primary node."`"        
        stop_db2

        # Kill db2fmd so it unlocks files on the mirror drive
        debug "Kill db2fmd."
        kill_db2fmd
        
        # Create the AM GUI file from Tivoli GUID file
        /opt/tivoli/guid/tivguid -Show | grep Guid |  cut -d':' -f2 | sed -e 's/\.//g'| sed -e 's/[a-z]/\u&/g' | tr -d \\n > /etc/ibm/tivoli/AM.guid
                
        crt_tsa_model       
        
        debug "Append to config file"
        append_string_to_cfg_file "--node-order=$HOST,$node"
        append_to_cfg_file $CMD_PARMS
        
        record_cfg_status "`eval_gettext "Synchronizing configuration with secondary node."`"        
        send_conf_parms
        RC=$?
                        
        # Add the secondary node to drbd primary so we can fail over from this point on.
        add_drbd_secondary_node
    
        # Bring Director Online
        sm_msg "`eval_gettext "Done creating the resources."`"
    
        debug "Bring Director Online"
        start_all_rg_online        


        run_conf_plugins conf end
        
        debug "--- Configuration is done ---"
        rm_cfg_status_file
        rm_cfg_user_file
        
        exit $RC
    fi
    
}


###################################
## Main code                     ##
###################################

if [ $# -eq 0 ]; then
    echo_help
    exit 0
fi

proc_parms $*

##
## There are only a few ways this command can be called.
## Run down through each and process them.
##

# Take care of help first
proc_help
# If tsa is specified then create the tsa model
proc_tsa
###
if [ -z "$tsa" ]; then
    # If node is specified then they are doing initial setup
    proc_node
fi
###
if [ -z "$tsa" ]; then
    # DHCP
    proc_dhcp_on_off
fi
# If dns is specified then process DNS
proc_dns
# if syslog is specified then process syslog
proc_syslog
# If ntp is specified then process NTP
proc_ntp
# If authentication_mech is specified then sync authentication mechanisms
proc_authentication_mech 
# If users is specified then sync users
proc_users
# If firewall is specified then sync the firewall
proc_firewall
# If update-interfaces is specified the update the firewall interfaces
proc_update_interfaces
# If prep is specified then prep the nodes
proc_prep
# If drbd is specified then create the conf file
proc_drbd
# If terminate is specified then terminate the HA environment
proc_term

exit 0

