#!/bin/bash
# Licensed Materials - Property of IBM
# 
# 09N6996
#
# (C) Copyright IBM Corp. 2009, 2010 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.

PATH=/usr/bin:$PATH
PROGNAME=$(basename $0)
PROGPATH=$(dirname $0)

INSTALL_DIR=/opt/ibm/director
CMD_DIR=$INSTALL_DIR/ha/bin/priv
COMMON_CODE=/opt/ibm/director/ha/bin/include/smhacommon.sh

if [ -f $COMMON_CODE ]; then
    source $COMMON_CODE
else
    echo "$PROGNAME:Common code not found."
    exit 1
fi
#############################################################

PARM_FLAGS="F finish s setup d dhcp-init"
PARM_MULTI=""
PARM_TRANSLATION="F finish \
s setup \
p password \
d dhcp-init" 
PARM_SINGLES=""
PARM_VALID=$PARM_TRANSLATION
PARM_DONT_SAVE="help password"

load_common_code $INSTALL_DIR/ha/bin/include/smhaparms.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhadrbd.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhadir.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhadb2.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhamodel.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhacim.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhaplugin.sh
load_common_code $INSTALL_DIR/ha/bin/include/smharsrc.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhadhcp.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhaagent.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhaerrcode.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhainitcom.sh


###################################
## Functions                     ##
###################################
proc_parms () {
    
    # Parse the parameters
    # The result should be that variables are set for each parameter and
    # CMD_PARMS will contain the list of parameters processed
    # If a file is specified it will also read the file.
    parse_all_parms $*    
        
}


#####################################################
# Process the setup parameter
#####################################################
proc_setup () {
    
    # if the secondary parameter was specified then handle it.
    if [ -n "$setup" ]; then
           
        ensure_parms "password"
        
        remove_director_autostart
        save_dirstartservice_status
        remove_dirstartservice
        
        record_dhcp_autostart
        
        remove_dhcp_autostart
        
        # Comment out the DB2 FMC and turn off auto start
        debug "Turn off db2 autostart"
        comment_fmc
        db2auto_off

        # Settings in /etc/inittab were changed. Poke init to read the new config
        init q

        # Take down Director
        debug "Stop director"
        $SMHA_SCRIPTS/tsa/smhatsadir.sh stop
        
        # Stop db2
        debug "Stop db2"
        stop_db2

        # Kill db2fmd so it unlocks files on the mirror drive
        debug "kill db2fmd"
        kill_db2fmd
        
        debug "Unmanage agent"            
        unmanage_agent
        debug "Reset management servers"            
        reset_mgmt_servers
        debug "Register agent with agent manager"                    
        register_agent_with_am ${password}
                        
        rm -rf ${SMHA_CONFIGDIR}/.haConfigRemoved >/dev/null 2>&1
        RC=$?
        
        debug "Secondary init completed with rc=$RC"
        exit $RC
    fi
    
}

#####################################################
# Process the finish parameter
#####################################################
proc_finish () {
    
    # if the finish parameter was specified then handle it.
    if [ -n "$finish" ]; then

        add_ha_to_inittab

        # Create the flag file so when init q is run in the next step
        # the smhamaint.sh script doesn't do anything.  It needs to be
        # run now though, so it doesn't run later when some other
        # script calls init q
        create_stop_maint_flagfile

        #Settings in /etc/inittab were changed. Poke init to read the new config
		init q

        updt_all_agntmgr_files_secondary
        
        run_conf_plugins conf end
                                
        RC=$?
        
        rm_cfg_status_file
        rm_cfg_user_file
        
        exit $RC
    fi
    
}

#####################################################
# Process the dhcp-init parameter
#####################################################
proc_dhcp_init () {
    
    # if the dhcp_init parameter was specified then handle it.
    if [ -n "$dhcp_init" ]; then
                                        
        dhcp_secondary_nonshared_init
        RC=$?
        
        exit $RC
    fi
    
}

###################################
## Main code                     ##
###################################

proc_parms $*

##
## There are only a few ways this command can be called.
## Run down through each and process them.
##

# If setup is specified then do initial setup on the secondary node
proc_setup
# If finish is specified then do final setup on the secondary node
proc_finish

proc_dhcp_init

exit 0

