#!/bin/bash
# Licensed Materials - Property of IBM
# 
# 09N6996
#
# (C) Copyright IBM Corp. 2010, 2010 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.

PATH=/usr/bin:$PATH
PROGNAME=$(basename $0)
PROGPATH=$(dirname $0)

INSTALL_DIR=/opt/ibm/director
CMD_DIR=$INSTALL_DIR/ha/bin/priv
COMMON_CODE=/opt/ibm/director/ha/bin/include/smhacommon.sh

if [ -f $COMMON_CODE ]; then
    source $COMMON_CODE
else
    echo "$PROGNAME:Common code not found."
    exit 1
fi
#############################################################

shutdown_wo_failover () {
	local RC=0
	
	is_configured
	if [ $? -eq 1 ]; then
		# HA is configured, check if this node is active
		debug "shutdown_wo_failover"
		is_passive
	    if [ $? -eq 0 ]; then
	    	# This is the active node
	    	debug "shutdown_wo_failover -active node."
            create_exclude_peer_flagfile
            # Exclude our peer, so only the floaters will be offline
	    	exclude_peer_node
	    fi
        set_crit_rsrc_protection $RSCT_CRIT_RSRC_PROTECT__OFF           
	fi
	
	return $RC
}

shutdown_w_failover () {
    local RC=0
    
    is_configured
    if [ $? -eq 1 ]; then
        # HA is configured, check if this node is active
        is_passive
        if [ $? -eq 0 ]; then
        
            get_remote_rsct_node_name
            node=$REMOTE_RSCT_NODE_NAME
            is_node_online $node
            if [ $? -eq 1 ]; then 
                # This is the active node, and we want a fail over
                debug "shutdown_w_failover -active node."
                
                create_shutdown_halt_flagfile
                # Remove this node from the domain so all the resources are shutdown
                exclude_node_from_domain                        
                set_crit_rsrc_protection $RSCT_CRIT_RSRC_PROTECT__OFF
                sleep 20
                            
                # Wait until we're passive
                local loop_count=0
                while true; do
		            is_passive
		            if [ $? -eq 1 ]; then
	                	break
		            else
		                let "loop_count=loop_count + 1"
		                debug "Waiting for node to become passive: $loop_count"
		                if [ $loop_count -ge 200 ]; then
		                	debug "Timed out waiting to become passive."
		                	break	            	
		                else
		                   sleep 5
	                    fi
                    fi
                done
            else
                debug "shutdown_w_failover -active node, but the other node is offline."
            fi
        else
            set_crit_rsrc_protection $RSCT_CRIT_RSRC_PROTECT__OFF        
        fi
    fi
    
    return $RC
	
}


###################################
## Main code                     ##
###################################
RC=0
debug "smhaspi called with $* "

while getopts aAbeiIsf flag;do
    case $flag in
        a) turn_off_automation ;;
        A) turn_on_automation ;;
        b) create_automation_flagfile ;;
        #################################
        e) exclude_peer_node ;;
        i) include_peer_node ;;
        I) create_exclude_peer_flagfile ;;
        #################################       
        s) #shutdown w/o failover
            shutdown_wo_failover
            RC=$?
        ;;
        f) #shutdown w/ failover
            shutdown_w_failover
            RC=$?
        ;;
    esac
done

debug "smhaspi exit; RC=$RC"
exit $RC
