#!/bin/bash
# Licensed Materials - Property of IBM
# 
# 09N6996
#
# (C) Copyright IBM Corp. 2009, 2010 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.

PROGNAME=$(basename $0)
PROGPATH=$(dirname $0)

INSTALL_DIR=/opt/ibm/director
CMD_DIR=$INSTALL_DIR/ha/bin/priv
COMMON_CODE=/opt/ibm/director/ha/bin/include/smhacommon.sh

if [ -f $COMMON_CODE ]; then
    source $COMMON_CODE
else
    echo "$PROGNAME:Common code not found."
    exit 1
fi


print_header () {
	echo -e " ==============================================="
	echo -e " |" `date +%F\ %T` "  HA Resource Status    |"
    echo -e " ==============================================="
}

print_status_for_resource () {
    local name=$1

    local tmpf=`mktemp /tmp/smha_stat.XXXXXX`  || fatal_error "`eval_gettext "Could not make temporary file"`"
    grep TSA_ACTION $LOG_FILE | grep $name >$tmpf
    local last_status=990993
    while read -r line
    do

        local actiona=${line##*TSA_ACTION:}
        local action=${actiona%%:*}

        local acta_rc=${line%:}
        local action_rc=${acta_rc##*:}
        local date_time=${line:0:19}
        local seqnum=${line:20:4}
        
        case "$action" in
            status*)
                if [ $last_status -ne $action_rc ]; then
                    echo " $date_time $seqnum $name $action $action_rc"
                    last_status=$action_rc
                fi
            ;;
            start*)
                    echo " $date_time $seqnum $name $action $action_rc"
            ;;
            stop*)
                    echo " $date_time $seqnum $name $action $action_rc"
            
            ;;
        esac
    done < $tmpf    
    rm $tmpf

}

print_quick_status_for_resource () {
    local name=$1

    local tmpf=`mktemp /tmp/smha_pqstat.XXXXXX`  || fatal_error "`eval_gettext "Could not make temporary file"`"
    grep TSA_ACTION $LOG_FILE | grep $name | sort -r >$tmpf
    while read -r line
    do

        local actiona=${line##*TSA_ACTION:}
        local action=${actiona%%:*}

        local acta_rc=${line%:}
        local action_rc=${acta_rc##*:}
        local date_time=${line:0:19}
        local seqnum=${line:20:4}
        
        case "$action" in
            status*)
                    echo " $date_time $seqnum $name $action $action_rc"
                    rm $tmpf
                    return
            ;;
        esac
    done < $tmpf    
    rm $tmpf

}


get_names () {
    if [ -z "$single_name" ]; then
        names=`grep TSA_ACTION $LOG_FILE| cut -d: -f7 | sort -u `
    else
        names="$single_name"
    fi
}

print_longer_status () {
    
    get_names

    while true; do
        clear
        print_header

	    for name in $names
	    do
	        echo " ========= $name ==========="
	        print_status_for_resource $name
	    done
        if [ $refresh -eq 1 ]; then
            sleep 7
            get_names
        else
            return
        fi
    done
	    
}

print_quick_status () {
    
    get_names

    while true; do
    	clear
    	print_header
    	
	    for name in $names
	    do
	        echo " ========= $name ==========="
	        print_quick_status_for_resource $name
	    done
	    if [ $refresh -eq 1 ]; then
	    	sleep 7
	    	get_names
	    else
	        return
	    fi
    done
}

chk_log_file () {

    while true; do
        if [ -f $LOG_FILE ]; then
            return
        fi
    	clear
    	print_header
        echo ""
        echo "HA log file does not exsit. File:$LOG_FILE"    	
	    if [ $refresh -eq 1 ]; then
	    	sleep 10
	    else
	        return
	    fi
    done

}

show_help () {
    echo "smhastat.sh [r][q][s name][f path][h]"
    echo " -r   refresh"
    echo " -q   quick listing"
    echo " -s   user selected list of resource names"
    echo " -f   user selected log file"
    echo " -h   show this help"
    exit 0
}

#################################################
#  Main code
#################################################
quickStatus=0
refresh=0
LOG_FILE=$LOG

while getopts qrs:hf: flag;do
    case $flag in
        q) quickStatus=1 ;;
        r) refresh=1 ;;
        s) single_name=$OPTARG ;;
        f) LOG_FILE=$OPTARG ;;
        h) show_help; exit ;;
    esac
done

chk_log_file
if [ "$quickStatus" == "1" ]; then
	print_quick_status
else 
    print_longer_status
fi

