#!/bin/bash
# Licensed Materials - Property of IBM
# 
# 09N6996
#
# (C) Copyright IBM Corp. 2009, 2010 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.

PATH=/usr/bin:$PATH
PROGNAME=$(basename $0)
PROGPATH=$(dirname $0)

#### Standard code to find an pull in all sm common code ####
INSTALL_DIR=/opt/ibm/director
CMD_DIR=$INSTALL_DIR/ha/bin/
COMMON_CODE=/opt/ibm/director/ha/bin/include/smhacommon.sh

if [ -f $COMMON_CODE ]; then
    source $COMMON_CODE
else
    echo "$PROGNAME:Common code not found."
    exit 1
fi
#############################################################
load_common_code $INSTALL_DIR/ha/bin/include/smhaparms.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhaerrcode.sh

PARM_FLAGS=""
PARM_MULTI=""
PARM_TRANSLATION="R resource"
PARM_SINGLES="R resource"
PARM_VALID=$PARM_TRANSLATION
INIT_SCRIPT=$SMHA_SCRIPTS/priv/smhainit.sh
###################################
## Functions                     ##
###################################
proc_parms () {
	
	# Parse the parameters
	# The result should be that variables are set for each parameter and
	# CMD_PARMS will contain the list of parameters processed
    parse_all_parms $*
    
}

#####################################################
# Process the resource parameter
#####################################################
proc_resource () {
    
    if [ -n "$resource" ]; then
        # Run through each resource that is supported
        case "$resource" in
            firewall|FIREWALL )
                $INIT_SCRIPT --firewall
                RC=$?                
            ;;
            users|USERS )
                $INIT_SCRIPT --users
                RC=$?
            ;;
            dns|DNS )
                $INIT_SCRIPT --dns
                RC=$?
            ;;
            ntp|NTP )
                $INIT_SCRIPT --ntp
                RC=$?
            ;;
            authmech|AUTHMECH )
                $INIT_SCRIPT --authentication-mech
                RC=$?
            ;;
            fwinterfaces|FWINTERFACES )
                $INIT_SCRIPT --update-interfaces
                RC=$?
            ;;
            syslog|SYSLOG )
                $INIT_SCRIPT --syslog
                RC=$?
            ;;
            
            *)
                debug "Sync request of unknown service: $resource"
                RC=$BAD_PARAMETER_VALUE
            ;;
        esac
        
        exit $RC
    fi
    
}

###################################
## Main code                     ##
###################################
proc_parms $*

# Check if HA is configured, if not exit
is_configured
if [ $? -eq 1 ]; then 
    # Process the resource parameter
    proc_resource
else
    exit $HA_ENVIRONMENT_NOT_CONFIGURED
fi

exit 0


