#!/bin/bash
# Licensed Materials - Property of IBM
# 
# 09N6996
#
# (C) Copyright IBM Corp. 2009, 2010 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.

PROGNAME=$(basename $0)
PROGPATH=$(dirname $0)
PATH=$PATH:/usr/smrshbin/
INSTALL_DIR=/opt/ibm/director
CMD_DIR=$INSTALL_DIR/ha/bin/priv
COMMON_CODE=/opt/ibm/director/ha/bin/include/smhacommon.sh

if [ -f $COMMON_CODE ]; then
    source $COMMON_CODE
else
    echo "$PROGNAME:Common code not found."
    exit 1
fi
#############################################################
## Run termination processing for HA
#############################################################
load_common_code $INSTALL_DIR/ha/bin/include/smhaparms.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhadrbd.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhadb2.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhadhcp.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhadir.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhaerrcode.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhaplugin.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhaagent.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhacim.sh

PARM_FLAGS="F force F force r reset R restore"
PARM_MULTI=""
PARM_TRANSLATION="F force r reset R restore"
PARM_SINGLES=""
PARM_VALID=$PARM_TRANSLATION


###################################
## Functions                     ##
###################################
proc_parms () {
    
    # Parse the parameters
    # The result should be that variables are set for each parameter and
    # CMD_PARMS will contain the list of parameters processed
    # If a file is specified it will also read the file.
    parse_all_parms $*
    
    # this is only for debug
    #list_parm_vars  
    
    
}

ensure_domain_is_deleted () {


    if [ -z "$force" ]; then

    	local i
    	for i in {1..180}
        do
            lsrpdomain -x -l $TSA_DOMAIN &> /dev/null
            if [ $? -eq 6 ]; then
            	return
        	else
                debug "Domain not deleted.  Wait."
                sleep 10
            fi
        done
    else
        remove_n_wait_for_domain 
    fi    	

}


###################################
## Main code                     ##
###################################
RC=0

proc_parms $*

debug "Running final termination code."

# Mark that termination has started
record_starting_termination

# Make sure the domain no longer exists
ensure_domain_is_deleted

comment_out_ha_inittab

# In the normal case this should already have been done
/etc/rc.d/smhamq stop
/etc/rc.d/smhadm stop

# Remove any MQ data
rm -rf $SMHA_DIR/persist/*

# Mount the mirrors
mount_mirrors

remove_dir_ha_props
dirc_use_settings active

#rm -rf /etc/ibm/tivoli/AM.guid

# load the agent manager address
get_agntmgr_addr
# map the floating address to a static    
find_static_address_for_floater $agent_manager_address
static_address=$RETURN_STATIC_ADDR

# Before updating the file, stop the agent
endpoint_dir=$INSTALL_DIR/agent/runtime/agent/bin
$endpoint_dir/endpoint.sh stop &> /dev/null
updt_agent_endpt_prop $static_address

if [ -n "$reset" ]; then
	debug "not starting CAS agent since this node needs to be reinstalled"
else
	# Start the CAS agent back up
	debug "starting db2"
	$endpoint_dir/endpoint.sh start &> /dev/null
fi

replace_agentmgr_ARShost_addr $static_address
turn_on_address_change_processing
set_am_mnic_on

terminate_dhcpd__restore_orig_config

# Re-enable any applications
if [ -n "$reset" ]; then
	debug "not turning director autostart or other apps back on since this node needs to be reinstalled"
else
	restore_dirstartservice_status
	restore_dhcp_if_needed
	# Re-enable db2
	uncomment_fmc
	db2auto_on
fi

run_conf_plugins unconf end

# Remove the configuration files
rm -rf $SMHA_DIR/config/*

# Backup the drbd config, just in case
if [ -f /etc/drbd.conf ]; then
	if [ -f /etc/drbd.conf.bkup ]; then
		rm /etc/drbd.conf.bkup
	fi
    mv /etc/drbd.conf /etc/drbd.conf.bkup
fi

if [ -n "$reset" ]; then

#	debug "starting db2"
#	/bin/su - db2inst1 -c  'db2stop force >/dev/null 2>&1; db2start >/dev/null 2>&1'
#	db2_list=`/bin/su - db2inst1 -c "db2 list db directory"`
#	debug "db2 list:$db2_list"
	
#	debug "running smreset"
#	export PATH=$PATH:/home/db2inst1/sqllib/bin:/usr/lib/mit/bin:/usr/lib/mit/sbin:/opt/ibm/director/bin:/opt/ibm/director/tpm/tools
#    /opt/ibm/director/bin/smreset 2>&1 >$LOG_DIR/smreset.log <<<yes
#    debug "reset done"

	# instead of running smreset (since it doesn't reset agent manager data completely), we need to
	# disable director (agent and server) so that the user is forced to reinstall

	# Add a flag file indicating director isn't usable due to HA unconfigure.  /usr/smrshbin/smstart will check this flag file
	echo "Director Server, Agent Manager, and CAS Agent should not be used on this system because they are in an inconsistent state due to High Availability being unconfigured" > ${SMHA_CONFIGDIR}/.haConfigRemoved
	debug "Done with HA termination."
else
	# Don't try restarting resources, just reboot
	debug "Done with HA termination. Rebooting..."
	/usr/smrshbin/smshutdown -t now -r
	exit $RC
fi


