#!/bin/bash
# Licensed Materials - Property of IBM
# 
# 09N6996
#
# (C) Copyright IBM Corp. 2009, 2011 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
#
PROGNAME=$(basename $0)
PROGPATH=$(dirname $0)
#### Use a special path for TSA monitoring scripts
PATH=/bin:/usr/bin:/sbin:/usr/sbin:/usr/sbin/rsct/bin
export PATH

#### Standard code to find an pull in all sm common code ####
INSTALL_DIR=/opt/ibm/director
CMD_DIR=$INSTALL_DIR/ha/bin/tsa
COMMON_CODE=/opt/ibm/director/ha/bin/include/smhacommon.sh

if [ -f $COMMON_CODE ]; then
    source $COMMON_CODE
else
    echo "$PROGNAME:Common code not found."
    exit 1
fi
#############################################################
load_common_code $INSTALL_DIR/ha/bin/include/smhadrbd.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhaplugin.sh

DRBD_LOCK_FILE=$SMHA_CONFIGDIR/lock-
NO_MIRRORS_FILE=$SMHA_CONFIGDIR/.drbd_nomirrors_flag
SPLIT_BRAIN_FILE=$SMHA_CONFIGDIR/.drbd_splitbrain_flag

# Keep track if we stopped a resource temporarily
RESOURCE_STOPPED=

#################################################
#  Functions
#################################################
get_drbd_version ()  {
    if [ -z "$DRBD_VER" ]; then
       local kernel_version=`uname -r`
       DRBD_VER=`grep -m 1 "drbd" /lib/modules/"$kernel_version"/modules.dep | awk -F/ '{print $NF}' | cut -d'.' -f 1`
    fi
		
	debug "drbd version is:$DRBD_VER"	
}

modprobe_drbd () {
	get_drbd_version
	local mp=`modprobe $DRBD_VER 2>&1`
	debug "modprobe $DRBD_VER  output:$mp"
}

modprobe_r_drbd () {
	get_drbd_version
	
	local try_count=0
	while true; do
		local mp=`modprobe -r $DRBD_VER 2>&1`
		local mp_rc=$?
		debug "Try:$try_count modprobe -r $DRBD_VER  rc:$mp_rc output:$mp"
		if [ $mp_rc -eq 0 ]; then
			break
		else
		    let "try_count=try_count + 1"
		    if [ $try_count -le 8 ]; then
		       sleep 5
		    else
		       break
	        fi
		fi
	done
	
	
}


clean_up () {
#	debug "Cleanup start"
    if [ -n "$RESOURCE_STOPPED" ]; then
       debug "Cleanup calling plugin for $RESOURCE_STOPPED"
	   run_drbd_plugins start $RESOURCE_STOPPED
    fi
    
    drbd_mutex_off
    
#    debug "Cleanup done"	
}

trap "clean_up" SIGTERM EXIT 


clear_invalidate_flag () {
    local resourcename=$0
    
    rm -f $SMHA_CONFIGDIR/$DRBD_INVALIDATE_FILE\.${resourcename}
}

check_invalidate_flag () {
	local resourcename=$0
	
	if [ -e $SMHA_CONFIGDIR/$DRBD_INVALIDATE_FILE\.${resourcename} ]; then
		
        # if we're already syncing then just ignore it
        is_syncing $name
        if [ $? -eq 0 ]; then
            #cmd_out=`drbdadm invalidate-remote all`
            cmd_out=`drbdadm invalidate-remote ${resourcename}`
            debug "Invalidating remote.  output:$cmd_out"
        fi  				
		clear_invalidate_flag $resourcename
		sleep 2
	fi
}

if_syncing_clear_flag () {
	local resourcename=$0
	
    if [ -e $SMHA_CONFIGDIR/$DRBD_INVALIDATE_FILE\.${resourcename} ]; then
        
        # if we're already syncing then just ignore it
        is_syncing $name
        if [ $? -eq 1 ]; then
            clear_invalidate_flag $resourcename
        fi
    fi
	
}

set_no_mirrors_flag () {
    touch $NO_MIRRORS_FILE
}

reset_no_mirrors_flag () {
    rm -f $NO_MIRRORS_FILE
}


is_primary () {
    running=`drbdadm state $name | grep "Primary/"`
        
    if [ "x$running" == "x"  ]
    then
        return 0
    else
        return 1
    fi

    return 0	
}


crt_metadata () {
    local mirror=${1}
    
    debug "Create meta-data for $mirror."
    drbdadm -f create-md ${mirror} 2>&1 <<-EOF
yes
yes
EOF
    debug "create-md rc:$?"

}

crt_metadata_if_needed () {
	local mirror=${1}
	
	chk_mirror=`grep $mirror $MD_FILE`
	if [ -z "$chk_mirror" ]; then
		# We didn't find this mirror in the file
		# Create the meta data
        crt_metadata $mirror		
		echo "$mirror" >> $MD_FILE
	fi
	
}

find_pids_with_openfiles () {
	# Pass in the path of interest
    of=${1}
    LOCK_PIDS=`LC_ALL=C lsof ${1} | grep -v COMMAND | awk '{print $2}'`
    
    local lpid
    for lpid in $LOCK_PIDS
    do
    	local psStr=`ps -aefww | grep " $lpid " | grep -v grep`
        debug "Found PID $lpid with lock on:$of"
        debug "PID $lpid ps:$psStr"
    done
    
}

kill_proc_for_mirror () {
    local mirror=${1}
    local force=${2}
    
    if [ "x$force" == "xY" ]; then
    	nine="-9"
    fi

    # First find the mount points for the mirror
    find_mountpoints_for_mirror $mirror
    pts="$MIRROR_MOUNT_PTS"
    for pts in ${pts}
    do
        debug "Find pids to kill $pts"       
        find_pids_with_openfiles $pts
        local lpids=$LOCK_PIDS
#        debug "List of pids to kill:$lpids:"       
        
        local lpid
        for lpid in ${lpids}
        do
        	debug "kill $nine process $lpid"
        	kill $nine $lpid
        done         
               
    done
    
    
}

kill_smstatus () {
	debug "kill smstatus called"
	local smstatus_pid=`ps -aefww | grep plugin.USMIStatusTest | grep -v grep | awk '{print $2}'`
	debug "smstatus pid:$smstatus_pid"
	if [ -n "$smstatus_pid" ]; then
		kill $smstatus_pid
	fi
	
	
}

umount_all_mirrors () {    
    # Get the list of mirrors        
    tmpf=`mktemp /tmp/smha_uam.XXXXXX`  || fatal_error "Could not make temp file"
    $GETMIRROR_CMD > $tmpf
    local mdev=
    local mmir=
    local mlab=
    while read -r line
    do
        mdev=`echo $line | cut -d' ' -f1` 
        mmir=`echo $line | cut -d' ' -f2`
        mlab=${mmir:1}
        # check if this mirror has a dependency.  If so, skip it.
       	check_for_dependent_mirror $mmir
       	if [ $? -eq 0 ]; then 
            # example: mdev=/dev/hda5, mmir=/mirror0, mlab=mirror0
            debug "umount_all_mirrors: first loop - calling umount_all $mmir $mlab"
        	umount_all $mmir $mlab
        else 
        	# skip it for now.  We will run through the list one more time after this to pick it up
        	debug "umount_all_mirrors: skipping $mmir for now - others are dependent on it."
        fi
    done < $tmpf    
    rm $tmpf
	
	# now run through the list one more time, but this time, only process the ones that
	# we skipped the first time.
	tmpf=`mktemp /tmp/smha_uam.XXXXXX`  || fatal_error "Could not make temp file"
    $GETMIRROR_CMD > $tmpf
	while read -r line
    do
        mdev=`echo $line | cut -d' ' -f1` 
        mmir=`echo $line | cut -d' ' -f2`
        mlab=${mmir:1}
    	check_for_dependent_mirror $mmir
       	if [ $? -eq 1 ]; then 
            # we skipped this one the last time through. process it.
            # example: mdev=/dev/hda5, mmir=/mirror0, mlab=mirror0
            debug "umount_all_mirrors: second loop - calling umount_all $mmir $mlab"
        	umount_all $mmir $mlab
        fi
    done < $tmpf    
    rm $tmpf      
}

crt_all_metadata () {
    # Get the list of mirrors        
    local tmpf=`mktemp /tmp/smha_cam.XXXXXX`  || fatal_error "Could not make temp file"
    $GETMIRROR_CMD > $tmpf
    while read -r line
    do
        local mdev=`echo $line | cut -d' ' -f1` 
        local mmir=`echo $line | cut -d' ' -f2`
        local mlab=${mmir:1}
        # mdev=/dev/hda5, mmir=/mirror0, mlab=mirror0
        crt_metadata_if_needed $mlab
         
    done < $tmpf    
    rm $tmpf
    
}

any_mirror_mounts () {

    # Get the list of mirrors        
    local tmpf=`mktemp /tmp/smha_amm.XXXXXX`  || fatal_error "Could not make temp file"
    $GETMIRROR_CMD > $tmpf
    while read -r line
    do
        local mdev=`echo $line | cut -d' ' -f1` 
        local mmir=`echo $line | cut -d' ' -f2`
        local mlab=${mmir:1}
        # mdev=/dev/hda5, mmir=/mirror0, mlab=mirror0
        any_mounts $mlab
        if [ $? -eq 1 ]; then
            rm $tmpf
        	return 1
        fi
         
    done < $tmpf    
    rm $tmpf

    return 0    
}

find_mounts_for_mirror () {
    local mirror=${1}
    
    MIRROR_MOUNTS=`LC_ALL=C mount | grep $mirror | cut -d' ' -f1 | grep -v spare`
    local mountout=`LC_ALL=C mount`
    debug "find_mounts_for_mirror: mount output: $mountout"
    debug "find_mounts_for_mirror:$MIRROR_MOUNTS:"

}

find_mountpoints_for_mirror () {
    local mirror=${1}
    
    MIRROR_MOUNT_PTS=`LC_ALL=C mount | grep $mirror | cut -d' ' -f3 | grep -v spare`
    debug "find_mountpoints_for_mirror:$MIRROR_MOUNTS:"

}


umount_mirror() {
    local mirror=${1}
    
    find_mounts_for_mirror $mirror

    for m in $MIRROR_MOUNTS
    do
    	debug "Unmounting: $m"
        cmd_out=`LC_ALL=C umount $m 2>&1`
        if [ -n "$cmd_out" ]; then
            debug "umount result:$cmd_out"
        fi
               
    done
}

any_mounts () {
    local mirror=${1}
    
    find_mounts_for_mirror $mirror    

    debug "any_mounts for mirror $mirror, mounts:$MIRROR_MOUNTS:"
    if [ -z "$MIRROR_MOUNTS" ]; then
    	return 0
    else
        return 1
    fi
}

umount_all() {
    local mountpt=${1}
    local name=${2}

    # Before attempting umount shutdown any resources that could holds locks
    run_drbd_plugins stop $mountpt
    RESOURCE_STOPPED=$mountpt

    local mounts_left=N
    local delay_time=0
    for i in {1..50}
    do
    	mounts_left=N
        # smstatus always needs to be killed
        kill_smstatus 

        # If we've been trying to gracefully shutdown for a while...
        # ...it's time to start killing processes
        if [ $i -gt 8 ]; then
        	if [ $i -gt 16 ]; then
        		force=Y
        	fi
            kill_proc_for_mirror $name $force
        fi
            
    	is_mounted $mountpt
   		if [ $? -eq 1 ]; then    			
			cmd_out=`LC_ALL=C umount $mountpt`
			debug "umount mirror $mountpt rc=$?: $cmd_out"
			mounts_left=Y
   		fi 
   		
        any_mounts $name
        if [ $? -eq 1 ]; then
            # Unmount any bind mounts        
            umount_mirror $name        
            mounts_left=Y            
        fi
        
        if [[ "$mounts_left" == "N" ]]; then
            break
        else
            let "delay_time = i + 1"
            if [ $delay_time -gt 5 ]; then
                deleay_time=5
            fi
            sleep $delay_time
        fi
        
    done
    # Start back up any resources.
    run_drbd_plugins start $mountpt
    RESOURCE_STOPPED=  
}

lock_base() {
    
    # See if the base is locked already by using a file
    if [ ! -e ${DRBD_LOCK_FILE}-base ]; then 
        
        local lock_out=`LC_ALL=C rgmbrreq -o lock IBM.Application:${DRBDB_RSRC_NAME}`
        debug "lock base : $lock_out"
        touch ${DRBD_LOCK_FILE}-base
    fi
}

unlock_base() {
    
    # Only unlock the base if there are no mirror locks
    if [ -e ${DRBD_LOCK_FILE}${mirror}* ]; then
    	debug "Cannot unlock base drbd, cause there are other locks."
    	return
    fi
        
    # See if the base is locked already by using a file
    if [ -e ${DRBD_LOCK_FILE}-base ]; then 
        
        local lock_out=`LC_ALL=C rgmbrreq -o unlock IBM.Application:${DRBDB_RSRC_NAME}`
        debug "unlock base: $lock_out"
        
        rm ${DRBD_LOCK_FILE}-base
    fi
}

lock_mirror() {
    local mirror=${1}
    
    if [ $DRBD_EQU -eq 0 ]; then
    
	    # See if the mirror is locked already by using a file
	    if [ ! -e ${DRBD_LOCK_FILE}${mirror} ]; then 
	        
	#        lock_base
	        
	        local lock_out=`LC_ALL=C rgmbrreq -o lock IBM.Application:${DRBDP_RSRC_NAME}-${mirror}`
	        debug "lock mirror $mirror: $lock_out"
	        touch ${DRBD_LOCK_FILE}${mirror}
	    fi
    fi
}

unlock_mirror() {
    local mirror=${1}

    if [ $DRBD_EQU -eq 0 ]; then
        
	    # See if the mirror is locked already by using a file
	    if [ -e ${DRBD_LOCK_FILE}${mirror} ]; then 
	        
	        rm ${DRBD_LOCK_FILE}${mirror}
	
	#        unlock_base
	        
	        local lock_out=`LC_ALL=C rgmbrreq -o unlock IBM.Application:${DRBDP_RSRC_NAME}-${mirror}`
	        debug "unlock mirror $mirror: $lock_out"
	        
	    fi
    fi
}

are_both_secondary () {
	
    local secondary=`drbdadm state $name | grep "Secondary/Secondary"`
    if [ "x$secondary" == "x"  ]
    then
        return 0
    else
        return 1
    fi

    return 0    
		
}

are_both_inconsistent () {
    
    local inconsistent=`drbdadm state $name | grep "Inconsistent/Inconsistent"`
    if [ "x$inconsistent" == "x"  ]
    then
        return 0
    else
        return 1
    fi

    return 0    
        
}

is_unconfigured () {
	
	status_out=`/etc/rc.d/drbd status`
    local unconfig=`echo "$status_out" | grep $name | grep Unconfigured`
#    debug "unconfig:$unconfig"
    if [ -z "$unconfig" ]; then
        return 0
    else
        return 1
    fi
                	
}

is_uptodate () {
	
    mirror_state=`drbd-overview | grep $name | awk '{print $4}'`
    case "$mirror_state" in         
        UpToDate/*)
            return 1
            ;;            
        *)
            return 0
            ;;
    esac            
}

check_and_handle_split_brain () {
	local mirror=$1
	
#hogdog3vm4:~ # /etc/init.d/drbd status
#drbd driver loaded OK; device status:
#version: 8.2.7 (api:88/proto:86-88)
#GIT-hash: a1b440e8b3011a1318d8bff1bb7edc763ef995b0 build by lmb@hermes, 2009-02-20 13:35:59
#m:res      cs          st                 ds                 p  mounted  fstype
#0:mirror0  StandAlone  Secondary/Unknown  UpToDate/DUnknown  -

    local drbdadm_out
    local drbdadm_rc
    #Check if we're in a StandAlone state
    local SA=`/etc/init.d/drbd status | grep $mirror | grep StandAlone`
    if [ -n "$SA" ]; then
    	# if it's secondary
    	local sec_SA=`echo $SA | grep Secondary/`
    	if [ -n "$sec_SA" ]; then
            debug "Found split brain $mirror --secondary"   
    		if [ -e $SPLIT_BRAIN_FILE ]; then
    			debug "saw split brain previously; recreating meta data"
                drbdadm_out=`drbdadm down ${mirror} 2>&1`
                drbdadm_rc=$?
                debug "split brain take down $mirror rc:$drbdadm_rc out:$drbdadm_out"
    			
                debug "split brain creating meta data for $mirror"
                crt_metadata $mirror
                    			
                drbdadm_out=`drbdadm up ${mirror} 2>&1`
                drbdadm_rc=$?
                debug "split brain bring up $mirror rc:$drbdadm_rc out:$drbdadm_out"
                
                rm -f $SPLIT_BRAIN_FILE
    		else
	    	    drbdadm_out=`drbdadm -- --discard-my-data connect $mirror`
	            drbdadm_rc=$?
	            debug "split brain drbdadm rc:$drbdadm_rc out:$drbdadm_out"
	            touch $SPLIT_BRAIN_FILE
            fi
    	else
    	    debug "Found split brain $mirror --primary"
    	    drbdadm_out=`drbdadm connect $mirror`
    	    drbdadm_rc=$?
    	    debug "split brain drbdadm rc:$drbdadm_rc out:$drbdadm_out"
        fi
    	
    else
        rm -f $SPLIT_BRAIN_FILE
    fi 
	
}

clear_drbd_upgrade_flag () {
    if [ -e $DRBD_UPGRADE_FLAG ]; then
        rm -f $DRBD_UPGRADE_FLAG
    fi
}

handle_locking () {
    # if done syncing then unlock
    is_syncing $name
    if [ $? -eq 1 ]; then
        lock_mirror $name
    else
        unlock_mirror $name
    fi	
}


drbd_mutex_on () {
	mutex_on drbd
}

drbd_mutex_off () {
	mutex_off drbd
}

#################################################
#  Main code
#################################################
RC=$TSA_UNKNOWN

# Get the parameters
action=${1} # start, stop, status
resource=${2} # base, primary, mounted
name=${3} # The disk label name (like mirror0)
device=${4} # The device {like /dev/sda5}
mountpt=${5} # The mount point {like /mirror0}
drbddev=${6} # The drbd device (like /dev/drbd0)

DRBDB_START_CMD="/etc/rc.d/drbd start"
DRBDP_START_CMD_OVERWRITE="drbdadm -- --overwrite-data-of-peer primary $name"
DRBDP_START_CMD="drbdadm primary $name"
DRBDB_STOP_CMD="/etc/rc.d/drbd stop"
DRBDP_STOP_CMD="drbdadm secondary $name"

debug_tsa "DRBD Action: $action Resource: $resource name: $name"

case ${action} in
start)
    record_tsa_action $action drbd-${resource}-$name "-"
################### START ############################
	case ${resource} in
	base)		
		modprobe_drbd
		
		umount_all_mirrors

        # Create the meta data if it doesn't already exist
        crt_all_metadata        
        
        any_mirror_mounts 
        if [ $? -eq 0 ]; then
            # Start up			    
	       $DRBDB_START_CMD
	       sm_sys_log "DRBD started"
	       RC=0
#        else
#            RC=${TSA_OFFLINE}   This causes the resource to go failed offline
        fi
	    ;;
    primary)    
        # Make this node the primary node
        
        for i in 1 2 3 4 5
        do
            if [ -e /proc/drbd ]; then
                break
            else
                debug "/proc/drbd not found. drbd base must not be online!"
                sleep 8
            fi
        done
        
        drbd_mutex_on
        
        mirrors_state=`drbd-overview  2>&1`
        debug "Start primary $drbddev, mirror states:\n$mirrors_state"
        cmd_out=`$DRBDP_START_CMD 2>&1`
        if [ $? -eq 17 ]; then
        	# 17 means both nodes are not uptodate        
            debug "primary $drbddev started with overwrite"
            cmd_out=`$DRBDP_START_CMD_OVERWRITE 2>&1`
        fi
        debug "primary $drbddev start rc:$? out:$cmd_out"

        # Check that we're really primary.        
        is_primary
        if [ $? -eq 1  ]; then 
	        # Lock it until it's sync'd 
	        lock_mirror $name
	        
	        #cale - added parameter of mirror name
	        check_invalidate_flag $name
	        clear_drbd_upgrade_flag
	        
	        sm_sys_log "DRBD $drbddev Primary started"
	        RC=0
        else
            RC=${TSA_OFFLINE}
        fi
        drbd_mutex_off
        ;;
    mounted)
    	is_mounted $drbddev
    	if [ $? -eq 0 ]; then    
    		debug "Start: Mounting $drbddev on $mountpt"
        	cmd_out=`LC_ALL=C mount -o rw $drbddev $mountpt`
        	debug "$cmd_out"

            run_drbd_plugins bindmounts $mountpt
        	
        	mount_mirror_bind_mounts $mountpt
    	else
    	   debug "Start: Already mounted $drbddev"    	   
    	fi
    	
    	run_drbd_plugins start $mountpt
    	
        sm_sys_log "DRBD Mounted started"
    	
        RC=0
    
        ;;        
    *)
        fatal_error "Bad parameter to $PROGNAME.  resource is: ${resource}"
        ;;        
    esac
    
    ;;
stop)
    record_tsa_action $action drbd-${resource}-$name "-"
################### STOP ############################
    case ${resource} in
    base)
    
        # Don't stop while syncing
        wait_while_syncing
        # It should be done syncing, but it might have timed out
        any_syncing
        if [ $? -eq 0 ]; then
            $DRBDB_STOP_CMD
        
            modprobe_r_drbd
        
            sm_sys_log "DRBD stopped"
        else
            debug "drbd base not stopped...sync in progress"
        fi        
        
        
	    RC=0
        ;;
    primary)
        # There shouldn't be any bind mounts left, but incase there is:
        umount_all $mountpt $name      
    
        # Make this node the secondary since we're stopping it
        cmd_out=`$DRBDP_STOP_CMD 2>&1`
        debug "primary stop rc:$? out:$cmd_out"
        
        sm_sys_log "DRBD Primary stopped"
        RC=0

        ;;
    mounted)
        umount_all $mountpt $name
        
		sm_sys_log "DRBD Mounted stopped"    
        RC=0
        ;;        
    *)
        fatal_error "Bad parameter to $PROGNAME. resource is: ${resource}"
        ;;        
    esac

    ;;

status)
################### STATUS ############################
    case ${resource} in
    base)    
	    status_out=`/etc/rc.d/drbd status`
	    statusrc=$?
	    debug_tsa "drbd status returned:$statusrc"
	    case $statusrc in
	        0) #Active
	            RC=${TSA_ONLINE}	        
	            any_mirrors=`echo $status_out | grep mirror`
	            if [ -z "$any_mirrors" ]; then
                    # No mirrors are listed!                                       
                    debug "status_out:$status_out"
                    if [ -e $NO_MIRRORS_FILE ]; then
                    	# We've seen this before, so unload drbd.
                        debug "No mirrors found in base status. Stopping base"
                        debug "status_out:$status_out"
                        $DRBDB_STOP_CMD
                        modprobe_r_drbd
                        reset_no_mirrors_flag
                        RC=${TSA_OFFLINE}
                    else
                        # Just record it and see if it happens again.
                        debug "No mirrors found in base status. Setting Flag file."
                        set_no_mirrors_flag
	                fi
                else
                    # We found a mirror so delete our flag file
                    reset_no_mirrors_flag
                fi
	            ;;
	        *) #Inactive
	            reset_no_mirrors_flag
	            RC=${TSA_OFFLINE}
	            ;;
	    esac
        ;;
    primary)
        is_primary
	    if [ $? -eq 0  ]; then
	    	###
	    	# We're the secondary
	    	#
	        # Check for a special case to unlock, 
	    	###
	    	
            # if neither node is Primary
            are_both_secondary
            if [ $? -eq 1 ]; then
            	debug "Both drbd nodes are secondary.  Handle any locks"
                if_syncing_clear_flag $name
                handle_locking	
            fi
            # If drbd is unconfigured
            is_unconfigured
            if [ $? -eq 1 ]; then
                debug "Drbd is unconfigured.  Handle any locks"
                handle_locking  
            fi
            # If we're uptodate, then check the locking
            is_uptodate 
            if [ $? -eq 1 ]; then
            	## This node is uptodate.  This should allow unlocks when the other node is down
            	## but we're up to date and we were the one that locked (because both were secondary)
                handle_locking  
            fi
                        		    	
	        RC=${TSA_OFFLINE}
	    else
	        RC=${TSA_ONLINE}
            # If we're the primary then do our duties
            	
            check_invalidate_flag $name

            handle_locking            	
        fi
        
        check_and_handle_split_brain $name
            
        ;;
    mounted)        
        is_mounted $mountpt
        if [ $? -eq 0  ]; then
            RC=${TSA_OFFLINE}
        else
            # Before we return status, check if it should be mounted
            is_primary
    	    if [ $? -eq 0  ]; then
    	    	debug "drbd mounted status: mounted when it should not be; unmounting"
                umount_all $mountpt $name	    
                RC=${TSA_OFFLINE}
            else
                RC=${TSA_ONLINE}
    	    fi
        
        fi
        
        ;;
    sync)
    
        mirror_state=`drbd-overview | grep ":$name " | awk '{print $4}'`
#        debug "drbd sync state for $name:$mirror_state"
        case "$mirror_state" in        	
        	UpToDate/*)        	   
                RC=${TSA_ONLINE}
                ;;
            Inconsistent/Inconsistent)
                # If both are inconsistent then either can be primary        	
                RC=${TSA_ONLINE}
                ;;
            Consistent/*)                
                # If this node is consistent then there isn't a connection to the other node           
                RC=${TSA_ONLINE}
                ;;
            Inconsistent/DUnknown)
                # Check if this is the upgrade case
                if [ -e $DRBD_UPGRADE_FLAG ]; then        	
                    RC=${TSA_ONLINE}
                else
            	    RC=${TSA_OFFLINE}
                fi
                ;;                
        	*)
        	    RC=${TSA_OFFLINE}
        	    ;;
        esac            
        
        ;;
    *)
        fatal_error "Bad parameter to $PROGNAME. resource is: ${resource}"
        ;;        
    esac

    ;;
cleanup)
################### CLEANUP ############################
    debug "DRBD ${resource} failed."
;;
*)
    fatal_error "Bad parameter to $PROGNAME. Action is:${action}"
    ;;        
    
esac

record_tsa_action $action drbd-${resource}-$name $RC

exit $RC
