#!/bin/bash
# Licensed Materials - Property of IBM
# 
# 09N6996
#
# (C) Copyright IBM Corp. 2009, 2010 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.

PROGNAME=$(basename $0)
PROGPATH=$(dirname $0)

INSTALL_DIR=/opt/ibm/director
CMD_DIR=$INSTALL_DIR/ha/bin/priv
COMMON_CODE=/opt/ibm/director/ha/bin/include/smhacommon.sh

if [ -f $COMMON_CODE ]; then
    source $COMMON_CODE
else
    echo "$PROGNAME:Common code not found."
    exit 1
fi
#############################################################
load_common_code $INSTALL_DIR/ha/bin/include/smhadhcp.sh

stop_syslog () {
    /etc/init.d/syslog status
    rc=$?
    if [ $rc -eq 0 ]; then
        # syslog is running, stop it
        debug "Stopping syslog"
        /etc/init.d/syslog stop
        # Record that we stopped it
        touch $SYSLOG_TRIP_FILE
    fi
}

restart_syslog () {
    # if we previously stoppped syslog
    if [ -e $SYSLOG_TRIP_FILE ]; then
        debug "Starting syslog"
        /etc/init.d/syslog start
        rm -f $SYSLOG_TRIP_FILE
    fi

}

stop_dhcpd () {
    /etc/init.d/dhcpd status
    rc=$?
    if [ $rc -eq 0 ]; then
        # dhcp is running, stop it
        debug "Stopping dhcpd"
        touch $SYSLOG_STOPPED_DHCP
        /etc/init.d/dhcpd stop
        kill_dhcp
    fi
}

start_dhcpd_if_stopped () {

    if [ -e $SYSLOG_STOPPED_DHCP ]; then
        /etc/init.d/dhcpd status
        rc=$?
        if [ $rc -ne 0 ]; then
            debug "Starting dhcpd because we stopped it"
            /etc/init.d/dhcpd start
        else
            debug "dhcpd was already started by someone else"
        fi
        rm -f $SYSLOG_STOPPED_DHCP
    fi
}


#### Main ########
method=${1}
mountpt=${2}

debug "$PROGNAME called with $method $mountpt"

if [[ "$mountpt" == "/mirror0" ]]; then

	case ${method} in
	stop)
	    stop_syslog
	    stop_dhcpd
	    ;;        
	start)
	    restart_syslog
	    start_dhcpd_if_stopped
	    ;;        
    bindmounts)
        # Nothing to do here                	
        echo "0" /dev/null
        ;;        

	*)
	    fatal_error "Bad parameter to $PROGNAME"
	    ;;        
	esac

fi
