#!/bin/bash
# Licensed Materials - Property of IBM
# 
# 09N6996
#
# (C) Copyright IBM Corp. 2010, 2010 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.

PATH=/usr/bin:$PATH
PROGNAME=$(basename $0)
PROGPATH=$(dirname $0)

INSTALL_DIR=/opt/ibm/director
CMD_DIR=$INSTALL_DIR/ha/bin/priv
COMMON_CODE=/opt/ibm/director/ha/bin/include/smhacommon.sh

if [ -f $COMMON_CODE ]; then
    source $COMMON_CODE
else
    echo "$PROGNAME:Common code not found."
    exit 1
fi
#############################################################
load_common_code $INSTALL_DIR/ha/bin/include/smhadrbd.sh

DEST_PRE=/spare
DEST_MIRROR0=$DEST_PRE/mirror0
DEST_OPT=$DEST_PRE/opt
DEST_VAR=$DEST_PRE/var
DEST_HOME=$DEST_PRE/home
DEST_ROOT=$DEST_PRE

DRBD_SOURCE=/etc/drbd.conf

RSCT_BKUP=/var/ct.backup

cpy_ha_config () {
	debug "cp -a $SMHA_CONFIGDIR ${DEST_ROOT}${SMHA_DIR}"
	cp -a $SMHA_CONFIGDIR ${DEST_ROOT}${SMHA_DIR}
	
	# Remove unwanted files
	rm -f ${DEST_ROOT}$MD_FILE
	
}

cpy_ha_persist () {
    debug "cp -a $SMHA_DIR/persist $DEST_ROOT/$SMHA_DIR"
    nice -n 10 cp -a $SMHA_DIR/persist $DEST_ROOT/$SMHA_DIR
        
}

cpy_ha_logs () {
    debug "cp -a $LOG_DIR $DEST_ROOT/$SMHA_DIR"
    nice -n 10 cp -a $LOG_DIR ${DEST_ROOT}${SMHA_DIR}
        
}

#cpy_tiv_guid () {
	# Not sure we need this since VMI copies /etc/ files....
#    if [ -f /etc/ibm/tivoli/AM.guid ]; then
#        debug "cp -a /etc/ibm/tivoli/AM.guid $DEST_ROOT/etc/ibm/tivoli/AM.guid"
#        cp -a /etc/ibm/tivoli/AM.guid $DEST_ROOT/etc/ibm/tivoli/AM.guid
#    fi
#        
#}


cpy_drbd_info () {
	debug "cp drbd info"
	
	on_host=$HOST
	
	getmirror_out=`$GETMIRROR_CMD`
	
    old_devices=`echo $getmirror_out | awk '{print $1}'`

    sed_line=    
    for old_device in $old_devices
    do 
    
	    is_d_dev=`echo $old_device | grep -e "/dev/[a-z]dd"`
	
	    if [ -n "$is_d_dev" ]; then
	        new_device=`echo $old_device | sed "s#dd#dg#g"`
	        
	    else
	        new_device=`echo $old_device | sed "s#dg#dd#g"`
	        
	    fi
	    
        debug "drbd.conf conversion; from $old_device to $new_device"

        sed_line_section="s#disk ${old_device}#disk ${new_device}#g"
        
        if [ -z "$sedline" ]; then
        	sed_line="$sed_line_section"
        else
            sed_line="${sed_line};${sed_line_section}"
        fi
    done

    debug "drbd sed line is: $sed_line"    
	sed "/on ${on_host}/ { N
	N
	N
	N
	N
	$sed_line}" /etc/drbd.conf >${DEST_ROOT}/etc/drbd.conf

    cp -a /etc/drbd.conf $DEST_ROOT/etc/drbd.conf.spare
}


bk_rsct () {
	
	if [ ! -d $RSCT_BKUP ]; then
        bkup_out=`nice -n 10 /usr/sbin/rsct/bin/ctbackup 2>&1` 
        # Remove the old copy
        rm -rf ${DEST_ROOT}${RSCT_BKUP}
        # Move the new copy to the spare
        nice -n 10 mv $RSCT_BKUP $DEST_VAR
        debug "RSCT BKUP: $bkup_out"
	fi
}

bk_ha_ssl_keys () {

    if [ -e /root/.ssh/authorized_keys2 ]; then
        cp -a /root/.ssh/authorized_keys2 /spare/root/.ssh/
    fi
}

###################################
## Main code                     ##
###################################
RC=0
debug " enter"

bk_rsct
bk_ha_ssl_keys
#cpy_tiv_guid

is_configured
if [ $? -eq 0 ]; then
    debug "HA is not configured."
else
    ## Run through the items to copy to the spare disk    
    cpy_ha_config
    cpy_ha_persist
    cpy_drbd_info

fi

# Copy the logs last so we have the most recent info
cpy_ha_logs

debug " exit; RC=$RC"
exit $RC
